@@ -6,6 +6,7 @@
 
 #define ARCH_X86_NAME "x86"
 #define ARCH_X64_NAME "x64"
+#define ARCH_MIPS_NAME "mips"
 #define ARCH_ARMv5_NAME "armv5"
 #define ARCH_ARMv6_NAME "armv6"
 #define ARCH_ARMv7_NAME "armv7a"
@@ -23,15 +24,15 @@
 #define PLATFORM_TEGRA_NAME "tegra"
 #define PLATFORM_TEGRA2_NAME "tegra2"
 #define PLATFORM_TEGRA3_NAME "tegra3"
-
+#define PLATFORM_TEGRA4_NAME "tegra4"
 
 class PackageInfo
 {
 public:
-    PackageInfo(const std::string& version, int platform, int cpu_id);
-    PackageInfo(const std::string& fullname, const std::string& install_path, const std::string& package_version = "0.0");
+    PackageInfo(int version, int platform, int cpu_id, std::string install_path = "/data/data/");
+    PackageInfo(const std::string& fullname, const std::string& install_path, std::string package_version = "0.0");
     std::string GetFullName() const;
-    std::string GetVersion() const;
+    int GetVersion() const;
     int GetPlatform() const;
     int GetCpuID() const;
     std::string GetInstalationPath() const;
@@ -41,7 +42,7 @@ public:
 
 protected:
     static std::map<int, std::string> InitPlatformNameMap();
-    std::string Version;
+    int Version;
     int Platform;
     int CpuID;
     std::string FullName;
@@ -49,4 +50,4 @@ protected:
     static const std::string BasePackageName;
 };
 
-#endif
\ No newline at end of file
+#endif
