@@ -4,7 +4,7 @@ using namespace std;
 
 bool PackageManagerStub::InstallPackage(const PackageInfo& package)
 {
-    InstalledPackages.push_back(PackageInfo(package.GetFullName(), "/data/data/" + package.GetFullName()));
+    InstalledPackages.push_back(package);
     return true;
 }
 
