@@ -28,9 +28,10 @@ PERF_TEST_P( Size_SrcDepth_DstChannels, merge,
         mv.push_back( Mat(sz, CV_MAKETYPE(srcDepth, 1)) );
         randu(mv[i], 0, 255);
     }
-    
+
     Mat dst;
-    TEST_CYCLE() merge( (vector<Mat> &)mv, dst );
+    int runs = (sz.width <= 640) ? 8 : 1;
+    TEST_CYCLE_MULTIRUN(runs) merge( (vector<Mat> &)mv, dst );
 
     SANITY_CHECK(dst, 1e-12);
 }
\ No newline at end of file
