@@ -42,31 +42,35 @@
 
 #include "precomp.hpp"
 
+#if defined _M_IX86 && defined _MSC_VER && _MSC_VER < 1700
+#pragma float_control(precise, on)
+#endif
+
 namespace cv
 {
- 
+
 /****************************************************************************************\
 *                     LU & Cholesky implementation for small matrices                    *
 \****************************************************************************************/
-    
+
 template<typename _Tp> static inline int
 LUImpl(_Tp* A, size_t astep, int m, _Tp* b, size_t bstep, int n)
 {
     int i, j, k, p = 1;
     astep /= sizeof(A[0]);
     bstep /= sizeof(b[0]);
-    
+
     for( i = 0; i < m; i++ )
     {
         k = i;
-        
+
         for( j = i+1; j < m; j++ )
             if( std::abs(A[j*astep + i]) > std::abs(A[k*astep + i]) )
                 k = j;
-        
+
         if( std::abs(A[k*astep + i]) < std::numeric_limits<_Tp>::epsilon() )
             return 0;
-        
+
         if( k != i )
         {
             for( j = i; j < m; j++ )
@@ -76,24 +80,24 @@ LUImpl(_Tp* A, size_t astep, int m, _Tp* b, size_t bstep, int n)
                     std::swap(b[i*bstep + j], b[k*bstep + j]);
             p = -p;
         }
-        
+
         _Tp d = -1/A[i*astep + i];
-        
+
         for( j = i+1; j < m; j++ )
         {
             _Tp alpha = A[j*astep + i]*d;
-            
+
             for( k = i+1; k < m; k++ )
                 A[j*astep + k] += alpha*A[i*astep + k];
-            
+
             if( b )
                 for( k = 0; k < n; k++ )
                     b[j*bstep + k] += alpha*b[i*bstep + k];
         }
-        
+
         A[i*astep + i] = -d;
     }
-    
+
     if( b )
     {
         for( i = m-1; i >= 0; i-- )
@@ -105,22 +109,22 @@ LUImpl(_Tp* A, size_t astep, int m, _Tp* b, size_t bstep, int n)
                 b[i*bstep + j] = s*A[i*astep + i];
             }
     }
-    
+
     return p;
 }
 
-    
+
 int LU(float* A, size_t astep, int m, float* b, size_t bstep, int n)
 {
     return LUImpl(A, astep, m, b, bstep, n);
 }
 
-    
+
 int LU(double* A, size_t astep, int m, double* b, size_t bstep, int n)
 {
     return LUImpl(A, astep, m, b, bstep, n);
 }
-   
+
 
 template<typename _Tp> static inline bool
 CholImpl(_Tp* A, size_t astep, int m, _Tp* b, size_t bstep, int n)
@@ -130,7 +134,7 @@ CholImpl(_Tp* A, size_t astep, int m, _Tp* b, size_t bstep, int n)
     double s;
     astep /= sizeof(A[0]);
     bstep /= sizeof(b[0]);
-    
+
     for( i = 0; i < m; i++ )
     {
         for( j = 0; j < i; j++ )
@@ -150,26 +154,26 @@ CholImpl(_Tp* A, size_t astep, int m, _Tp* b, size_t bstep, int n)
             return false;
         L[i*astep + i] = (_Tp)(1./std::sqrt(s));
     }
-    
+
     if( !b )
         return true;
-    
+
     // LLt x = b
     // 1: L y = b
     // 2. Lt x = y
-    
+
     /*
      [ L00             ]  y0   b0
      [ L10 L11         ]  y1 = b1
      [ L20 L21 L22     ]  y2   b2
      [ L30 L31 L32 L33 ]  y3   b3
-     
+
      [ L00 L10 L20 L30 ]  x0   y0
      [     L11 L21 L31 ]  x1 = y1
      [         L22 L32 ]  x2   y2
      [             L33 ]  x3   y3
     */
-    
+
     for( i = 0; i < m; i++ )
     {
         for( j = 0; j < n; j++ )
@@ -180,7 +184,7 @@ CholImpl(_Tp* A, size_t astep, int m, _Tp* b, size_t bstep, int n)
             b[i*bstep + j] = (_Tp)(s*L[i*astep + i]);
         }
     }
-    
+
     for( i = m-1; i >= 0; i-- )
     {
         for( j = 0; j < n; j++ )
@@ -191,22 +195,22 @@ CholImpl(_Tp* A, size_t astep, int m, _Tp* b, size_t bstep, int n)
             b[i*bstep + j] = (_Tp)(s*L[i*astep + i]);
         }
     }
-    
+
     return true;
 }
-    
-    
+
+
 bool Cholesky(float* A, size_t astep, int m, float* b, size_t bstep, int n)
 {
     return CholImpl(A, astep, m, b, bstep, n);
 }
-    
+
 bool Cholesky(double* A, size_t astep, int m, double* b, size_t bstep, int n)
 {
     return CholImpl(A, astep, m, b, bstep, n);
 }
 
-    
+
 template<typename _Tp> static inline _Tp hypot(_Tp a, _Tp b)
 {
     a = std::abs(a);
@@ -230,7 +234,7 @@ JacobiImpl_( _Tp* A, size_t astep, _Tp* W, _Tp* V, size_t vstep, int n, uchar* b
 {
     const _Tp eps = std::numeric_limits<_Tp>::epsilon();
     int i, j, k, m;
-    
+
     astep /= sizeof(A[0]);
     if( V )
     {
@@ -242,13 +246,13 @@ JacobiImpl_( _Tp* A, size_t astep, _Tp* W, _Tp* V, size_t vstep, int n, uchar* b
             V[i*vstep + i] = (_Tp)1;
         }
     }
-    
+
     int iters, maxIters = n*n*30;
-    
+
     int* indR = (int*)alignPtr(buf, sizeof(int));
     int* indC = indR + n;
     _Tp mv = (_Tp)0;
-    
+
     for( k = 0; k < n; k++ )
     {
         W[k] = A[(astep + 1)*k];
@@ -273,7 +277,7 @@ JacobiImpl_( _Tp* A, size_t astep, _Tp* W, _Tp* V, size_t vstep, int n, uchar* b
             indC[k] = m;
         }
     }
-    
+
     if( n > 1 ) for( iters = 0; iters < maxIters; iters++ )
     {
         // find index (k,l) of pivot p
@@ -290,7 +294,7 @@ JacobiImpl_( _Tp* A, size_t astep, _Tp* W, _Tp* V, size_t vstep, int n, uchar* b
             if( mv < val )
                 mv = val, k = indC[i], l = i;
         }
-        
+
         _Tp p = A[astep*k + l];
         if( std::abs(p) <= eps )
             break;
@@ -302,15 +306,15 @@ JacobiImpl_( _Tp* A, size_t astep, _Tp* W, _Tp* V, size_t vstep, int n, uchar* b
         if( y < 0 )
             s = -s, t = -t;
         A[astep*k + l] = 0;
-        
+
         W[k] -= t;
         W[l] += t;
-        
+
         _Tp a0, b0;
-        
+
 #undef rotate
 #define rotate(v0, v1) a0 = v0, b0 = v1, v0 = a0*c - b0*s, v1 = a0*s + b0*c
-        
+
         // rotate rows and columns k and l
         for( i = 0; i < k; i++ )
             rotate(A[astep*i+k], A[astep*i+l]);
@@ -318,14 +322,14 @@ JacobiImpl_( _Tp* A, size_t astep, _Tp* W, _Tp* V, size_t vstep, int n, uchar* b
             rotate(A[astep*k+i], A[astep*i+l]);
         for( i = l+1; i < n; i++ )
             rotate(A[astep*k+i], A[astep*l+i]);
-        
+
         // rotate eigenvectors
         if( V )
             for( i = 0; i < n; i++ )
                 rotate(V[vstep*k+i], V[vstep*l+i]);
-        
+
 #undef rotate
-        
+
         for( j = 0; j < 2; j++ )
         {
             int idx = j == 0 ? k : l;
@@ -351,7 +355,7 @@ JacobiImpl_( _Tp* A, size_t astep, _Tp* W, _Tp* V, size_t vstep, int n, uchar* b
             }
         }
     }
-    
+
     // sort eigenvalues & eigenvectors
     for( k = 0; k < n-1; k++ )
     {
@@ -369,10 +373,10 @@ JacobiImpl_( _Tp* A, size_t astep, _Tp* W, _Tp* V, size_t vstep, int n, uchar* b
                     std::swap(V[vstep*m + i], V[vstep*k + i]);
         }
     }
-    
+
     return true;
 }
-    
+
 static bool Jacobi( float* S, size_t sstep, float* e, float* E, size_t estep, int n, uchar* buf )
 {
     return JacobiImpl_(S, sstep, e, E, estep, n, buf);
@@ -382,8 +386,8 @@ static bool Jacobi( double* S, size_t sstep, double* e, double* E, size_t estep,
 {
     return JacobiImpl_(S, sstep, e, E, estep, n, buf);
 }
-    
-    
+
+
 template<typename T> struct VBLAS
 {
     int dot(const T*, const T*, int, T*) const { return 0; }
@@ -402,7 +406,7 @@ template<> inline int VBLAS<float>::dot(const float* a, const float* b, int n, f
     {
         __m128 a0 = _mm_load_ps(a + k), a1 = _mm_load_ps(a + k + 4);
         __m128 b0 = _mm_load_ps(b + k), b1 = _mm_load_ps(b + k + 4);
-        
+
         s0 = _mm_add_ps(s0, _mm_mul_ps(a0, b0));
         s1 = _mm_add_ps(s1, _mm_mul_ps(a1, b1));
     }
@@ -471,7 +475,7 @@ template<> inline int VBLAS<double>::dot(const double* a, const double* b, int n
     {
         __m128d a0 = _mm_load_pd(a + k), a1 = _mm_load_pd(a + k + 2);
         __m128d b0 = _mm_load_pd(b + k), b1 = _mm_load_pd(b + k + 2);
-        
+
         s0 = _mm_add_pd(s0, _mm_mul_pd(a0, b0));
         s1 = _mm_add_pd(s1, _mm_mul_pd(a1, b1));
     }
@@ -527,18 +531,18 @@ template<> inline int VBLAS<double>::givensx(double* a, double* b, int n, double
 #endif
 
 template<typename _Tp> void
-JacobiSVDImpl_(_Tp* At, size_t astep, _Tp* _W, _Tp* Vt, size_t vstep, int m, int n, int n1, double minval)
+JacobiSVDImpl_(_Tp* At, size_t astep, _Tp* _W, _Tp* Vt, size_t vstep,
+               int m, int n, int n1, double minval, _Tp eps)
 {
     VBLAS<_Tp> vblas;
     AutoBuffer<double> Wbuf(n);
     double* W = Wbuf;
-    _Tp eps = DBL_EPSILON*10;
     int i, j, k, iter, max_iter = std::max(m, 30);
     _Tp c, s;
     double sd;
     astep /= sizeof(At[0]);
     vstep /= sizeof(Vt[0]);
-    
+
     for( i = 0; i < n; i++ )
     {
         for( k = 0, sd = 0; k < m; k++ )
@@ -547,7 +551,7 @@ JacobiSVDImpl_(_Tp* At, size_t astep, _Tp* _W, _Tp* Vt, size_t vstep, int m, int
             sd += (double)t*t;
         }
         W[i] = sd;
-        
+
         if( Vt )
         {
             for( k = 0; k < n; k++ )
@@ -555,28 +559,28 @@ JacobiSVDImpl_(_Tp* At, size_t astep, _Tp* _W, _Tp* Vt, size_t vstep, int m, int
             Vt[i*vstep + i] = 1;
         }
     }
-    
+
     for( iter = 0; iter < max_iter; iter++ )
     {
         bool changed = false;
-        
+
         for( i = 0; i < n-1; i++ )
             for( j = i+1; j < n; j++ )
             {
                 _Tp *Ai = At + i*astep, *Aj = At + j*astep;
                 double a = W[i], p = 0, b = W[j];
-                
+
                 for( k = 0; k < m; k++ )
                     p += (double)Ai[k]*Aj[k];
-                
+
                 if( std::abs(p) <= eps*std::sqrt((double)a*b) )
-                    continue;           
-                
+                    continue;
+
                 p *= 2;
-                double beta = a - b, gamma = hypot((double)p, beta), delta;
+                double beta = a - b, gamma = hypot((double)p, beta);
                 if( beta < 0 )
                 {
-                    delta = (gamma - beta)*0.5;
+                    double delta = (gamma - beta)*0.5;
                     s = (_Tp)std::sqrt(delta/gamma);
                     c = (_Tp)(p/(gamma*s*2));
                 }
@@ -584,44 +588,26 @@ JacobiSVDImpl_(_Tp* At, size_t astep, _Tp* _W, _Tp* Vt, size_t vstep, int m, int
                 {
                     c = (_Tp)std::sqrt((gamma + beta)/(gamma*2));
                     s = (_Tp)(p/(gamma*c*2));
-                    delta = p*p*0.5/(gamma + beta);
                 }
-                
-                W[i] += delta;
-                W[j] -= delta;
-                
-                if( iter % 2 != 0 && W[i] > 0 && W[j] > 0 )
+
+                a = b = 0;
+                for( k = 0; k < m; k++ )
                 {
-                    k = vblas.givens(Ai, Aj, m, c, s);
-                    
-                    for( ; k < m; k++ )
-                    {
-                        _Tp t0 = c*Ai[k] + s*Aj[k];
-                        _Tp t1 = -s*Ai[k] + c*Aj[k];
-                        Ai[k] = t0; Aj[k] = t1;
-                    }
+                    _Tp t0 = c*Ai[k] + s*Aj[k];
+                    _Tp t1 = -s*Ai[k] + c*Aj[k];
+                    Ai[k] = t0; Aj[k] = t1;
+
+                    a += (double)t0*t0; b += (double)t1*t1;
                 }
-                else
-                {
-                    a = b = 0;
-                    for( k = 0; k < m; k++ )
-                    {
-                        _Tp t0 = c*Ai[k] + s*Aj[k];
-                        _Tp t1 = -s*Ai[k] + c*Aj[k];
-                        Ai[k] = t0; Aj[k] = t1;
-                        
-                        a += (double)t0*t0; b += (double)t1*t1;
-                    }
-                    W[i] = a; W[j] = b;
-                }
-                
+                W[i] = a; W[j] = b;
+
                 changed = true;
-                
+
                 if( Vt )
                 {
                     _Tp *Vi = Vt + i*vstep, *Vj = Vt + j*vstep;
                     k = vblas.givens(Vi, Vj, n, c, s);
-                    
+
                     for( ; k < n; k++ )
                     {
                         _Tp t0 = c*Vi[k] + s*Vj[k];
@@ -633,7 +619,7 @@ JacobiSVDImpl_(_Tp* At, size_t astep, _Tp* _W, _Tp* Vt, size_t vstep, int m, int
         if( !changed )
             break;
     }
-    
+
     for( i = 0; i < n; i++ )
     {
         for( k = 0, sd = 0; k < m; k++ )
@@ -643,7 +629,7 @@ JacobiSVDImpl_(_Tp* At, size_t astep, _Tp* _W, _Tp* Vt, size_t vstep, int m, int
         }
         W[i] = std::sqrt(sd);
     }
-    
+
     for( i = 0; i < n-1; i++ )
     {
         j = i;
@@ -659,24 +645,24 @@ JacobiSVDImpl_(_Tp* At, size_t astep, _Tp* _W, _Tp* Vt, size_t vstep, int m, int
             {
                 for( k = 0; k < m; k++ )
                     std::swap(At[i*astep + k], At[j*astep + k]);
-                
+
                 for( k = 0; k < n; k++ )
                     std::swap(Vt[i*vstep + k], Vt[j*vstep + k]);
             }
         }
     }
-    
+
     for( i = 0; i < n; i++ )
         _W[i] = (_Tp)W[i];
-    
+
     if( !Vt )
         return;
-    
+
     RNG rng(0x12345678);
     for( i = 0; i < n1; i++ )
     {
         sd = i < n ? W[i] : 0;
-        
+
         while( sd <= minval )
         {
             // if we got a zero singular value, then in order to get the corresponding left singular vector
@@ -715,24 +701,24 @@ JacobiSVDImpl_(_Tp* At, size_t astep, _Tp* _W, _Tp* Vt, size_t vstep, int m, int
             }
             sd = std::sqrt(sd);
         }
-        
+
         s = (_Tp)(1/sd);
         for( k = 0; k < m; k++ )
             At[i*astep + k] *= s;
     }
 }
 
-    
+
 static void JacobiSVD(float* At, size_t astep, float* W, float* Vt, size_t vstep, int m, int n, int n1=-1)
 {
-    JacobiSVDImpl_(At, astep, W, Vt, vstep, m, n, !Vt ? 0 : n1 < 0 ? n : n1, FLT_MIN);
+    JacobiSVDImpl_(At, astep, W, Vt, vstep, m, n, !Vt ? 0 : n1 < 0 ? n : n1, FLT_MIN, FLT_EPSILON*2);
 }
 
 static void JacobiSVD(double* At, size_t astep, double* W, double* Vt, size_t vstep, int m, int n, int n1=-1)
 {
-    JacobiSVDImpl_(At, astep, W, Vt, vstep, m, n, !Vt ? 0 : n1 < 0 ? n : n1, DBL_MIN);
+    JacobiSVDImpl_(At, astep, W, Vt, vstep, m, n, !Vt ? 0 : n1 < 0 ? n : n1, DBL_MIN, DBL_EPSILON*10);
 }
-    
+
 /* y[0:m,0:n] += diag(a[0:1,0:m]) * x[0:m,0:n] */
 template<typename T1, typename T2, typename T3> static void
 MatrAXPY( int m, int n, const T1* x, int dx,
@@ -742,8 +728,8 @@ MatrAXPY( int m, int n, const T1* x, int dx,
     for( i = 0; i < m; i++, x += dx, y += dy )
     {
         T2 s = a[i*inca];
-		j=0;
-		 #if CV_ENABLE_UNROLLED
+        j=0;
+         #if CV_ENABLE_UNROLLED
         for(; j <= n - 4; j += 4 )
         {
             T3 t0 = (T3)(y[j]   + s*x[j]);
@@ -772,18 +758,18 @@ SVBkSbImpl_( int m, int n, const T* w, int incw,
     int udelta0 = uT ? ldu : 1, udelta1 = uT ? 1 : ldu;
     int vdelta0 = vT ? ldv : 1, vdelta1 = vT ? 1 : ldv;
     int i, j, nm = std::min(m, n);
-    
+
     if( !b )
         nb = m;
-    
+
     for( i = 0; i < n; i++ )
         for( j = 0; j < nb; j++ )
             x[i*ldx + j] = 0;
-    
+
     for( i = 0; i < nm; i++ )
         threshold += w[i*incw];
     threshold *= eps;
-    
+
     // v * inv(w) * uT * b
     for( i = 0; i < nm; i++, u += udelta0, v += vdelta0 )
     {
@@ -791,7 +777,7 @@ SVBkSbImpl_( int m, int n, const T* w, int incw,
         if( (double)std::abs(wi) <= threshold )
             continue;
         wi = 1/wi;
-        
+
         if( nb == 1 )
         {
             double s = 0;
@@ -801,7 +787,7 @@ SVBkSbImpl_( int m, int n, const T* w, int incw,
             else
                 s = u[0];
             s *= wi;
-            
+
             for( j = 0; j < n; j++ )
                 x[j*ldx] = (T)(x[j*ldx] + s*v[j*vdelta1]);
         }
@@ -824,7 +810,7 @@ SVBkSbImpl_( int m, int n, const T* w, int incw,
         }
     }
 }
-    
+
 static void
 SVBkSb( int m, int n, const float* w, size_t wstep,
         const float* u, size_t ustep, bool uT,
@@ -853,10 +839,10 @@ SVBkSb( int m, int n, const double* w, size_t wstep,
                 b, (int)(bstep/sizeof(b[0])), nb,
                 x, (int)(xstep/sizeof(x[0])),
                 (double*)alignPtr(buffer, sizeof(double)), DBL_EPSILON*2 );
-}    
+}
 
 }
-    
+
 /****************************************************************************************\
 *                                 Determinant of the matrix                              *
 \****************************************************************************************/
@@ -893,7 +879,7 @@ double cv::determinant( InputArray _mat )
             AutoBuffer<uchar> buffer(bufSize);
             Mat a(rows, rows, CV_32F, (uchar*)buffer);
             mat.copyTo(a);
-            
+
             result = LU((float*)a.data, a.step, rows, 0, 0, 0);
             if( result )
             {
@@ -917,7 +903,7 @@ double cv::determinant( InputArray _mat )
             AutoBuffer<uchar> buffer(bufSize);
             Mat a(rows, rows, CV_64F, (uchar*)buffer);
             mat.copyTo(a);
-            
+
             result = LU((double*)a.data, a.step, rows, 0, 0, 0);
             if( result )
             {
@@ -953,17 +939,17 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
 
     size_t esz = CV_ELEM_SIZE(type);
     int m = src.rows, n = src.cols;
-    
+
     if( method == DECOMP_SVD )
     {
         int nm = std::min(m, n);
-        
+
         AutoBuffer<uchar> _buf((m*nm + nm + nm*n)*esz + sizeof(double));
         uchar* buf = alignPtr((uchar*)_buf, (int)esz);
         Mat u(m, nm, type, buf);
         Mat w(nm, 1, type, u.data + m*nm*esz);
         Mat vt(nm, n, type, w.data + nm*esz);
-        
+
         SVD::compute(src, w, u, vt);
         SVD::backSubst(w, u, vt, Mat(), _dst);
         return type == CV_32F ?
@@ -972,9 +958,9 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
             (((double*)w.data)[0] >= DBL_EPSILON ?
              ((double*)w.data)[n-1]/((double*)w.data)[0] : 0);
     }
-    
+
     CV_Assert( m == n );
-    
+
     if( method == DECOMP_EIG )
     {
         AutoBuffer<uchar> _buf((n*n*2 + n)*esz + sizeof(double));
@@ -982,7 +968,7 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
         Mat u(n, n, type, buf);
         Mat w(n, 1, type, u.data + n*n*esz);
         Mat vt(n, n, type, w.data + n*esz);
-        
+
         eigen(src, w, vt);
         transpose(vt, u);
         SVD::backSubst(w, u, vt, Mat(), _dst);
@@ -992,9 +978,9 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
         (((double*)w.data)[0] >= DBL_EPSILON ?
          ((double*)w.data)[n-1]/((double*)w.data)[0] : 0);
     }
-    
+
     CV_Assert( method == DECOMP_LU || method == DECOMP_CHOLESKY );
-    
+
     _dst.create( n, n, type );
     Mat dst = _dst.getMat();
 
@@ -1012,17 +998,39 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
                 double d = det2(Sf);
                 if( d != 0. )
                 {
-                    double t0, t1;
                     result = true;
                     d = 1./d;
-                    t0 = Sf(0,0)*d;
-                    t1 = Sf(1,1)*d;
-                    Df(1,1) = (float)t0;
-                    Df(0,0) = (float)t1;
-                    t0 = -Sf(0,1)*d;
-                    t1 = -Sf(1,0)*d;
-                    Df(0,1) = (float)t0;
-                    Df(1,0) = (float)t1;
+
+                    #if CV_SSE2
+                        if(USE_SSE2)
+                        {
+                            __m128 zero = _mm_setzero_ps();
+                            __m128 t0 = _mm_loadl_pi(zero, (const __m64*)srcdata); //t0 = sf(0,0) sf(0,1)
+                            __m128 t1 = _mm_loadh_pi(zero, (const __m64*)(srcdata+srcstep)); //t1 = sf(1,0) sf(1,1)
+                            __m128 s0 = _mm_or_ps(t0, t1);
+                            __m128 det =_mm_set1_ps((float)d);
+                            s0 =  _mm_mul_ps(s0, det);
+                            static const uchar CV_DECL_ALIGNED(16) inv[16] = {0,0,0,0,0,0,0,0x80,0,0,0,0x80,0,0,0,0};
+                            __m128 pattern = _mm_load_ps((const float*)inv);
+                            s0 = _mm_xor_ps(s0, pattern);//==-1*s0
+                            s0 = _mm_shuffle_ps(s0, s0, _MM_SHUFFLE(0,2,1,3));
+                            _mm_storel_pi((__m64*)dstdata, s0);
+                            _mm_storeh_pi((__m64*)((float*)(dstdata+dststep)), s0);
+                        }
+                        else
+                    #endif
+                        {
+                        double t0, t1;
+                        t0 = Sf(0,0)*d;
+                        t1 = Sf(1,1)*d;
+                        Df(1,1) = (float)t0;
+                        Df(0,0) = (float)t1;
+                        t0 = -Sf(0,1)*d;
+                        t1 = -Sf(1,0)*d;
+                        Df(0,1) = (float)t0;
+                        Df(1,0) = (float)t1;
+                        }
+
                 }
             }
             else
@@ -1030,17 +1038,41 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
                 double d = det2(Sd);
                 if( d != 0. )
                 {
-                    double t0, t1;
                     result = true;
                     d = 1./d;
-                    t0 = Sd(0,0)*d;
-                    t1 = Sd(1,1)*d;
-                    Dd(1,1) = t0;
-                    Dd(0,0) = t1;
-                    t0 = -Sd(0,1)*d;
-                    t1 = -Sd(1,0)*d;
-                    Dd(0,1) = t0;
-                    Dd(1,0) = t1;
+                    #if CV_SSE2
+                        if(USE_SSE2)
+                        {
+                            __m128d s0 = _mm_loadu_pd((const double*)srcdata); //s0 = sf(0,0) sf(0,1)
+                            __m128d s1 = _mm_loadu_pd ((const double*)(srcdata+srcstep));//s1 = sf(1,0) sf(1,1)
+                            __m128d sm = _mm_unpacklo_pd(s0, _mm_load_sd((const double*)(srcdata+srcstep)+1)); //sm = sf(0,0) sf(1,1) - main diagonal
+                            __m128d ss = _mm_shuffle_pd(s0, s1, _MM_SHUFFLE2(0,1)); //ss = sf(0,1) sf(1,0) - secondary diagonal
+                            __m128d det = _mm_load1_pd((const double*)&d);
+                            sm =  _mm_mul_pd(sm, det);
+
+                            static const uchar CV_DECL_ALIGNED(16) inv[8] = {0,0,0,0,0,0,0,0x80};
+                            __m128d pattern = _mm_load1_pd((double*)inv);
+                            ss = _mm_mul_pd(ss, det);
+                            ss = _mm_xor_pd(ss, pattern);//==-1*ss
+
+                            s0 = _mm_shuffle_pd(sm, ss, _MM_SHUFFLE2(0,1));
+                            s1 = _mm_shuffle_pd(ss, sm, _MM_SHUFFLE2(0,1));
+                            _mm_storeu_pd((double*)dstdata, s0);
+                            _mm_storeu_pd((double*)(dstdata+dststep), s1);
+                        }
+                        else
+                    #endif
+                        {
+                            double t0, t1;
+                            t0 = Sd(0,0)*d;
+                            t1 = Sd(1,1)*d;
+                            Dd(1,1) = t0;
+                            Dd(0,0) = t1;
+                            t0 = -Sd(0,1)*d;
+                            t1 = -Sd(1,0)*d;
+                            Dd(0,1) = t0;
+                            Dd(1,0) = t1;
+                        }
                 }
             }
         }
@@ -1049,27 +1081,28 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
             if( type == CV_32FC1 )
             {
                 double d = det3(Sf);
+
                 if( d != 0. )
                 {
-                    float t[9];
+                    double t[12];
+
                     result = true;
                     d = 1./d;
+                    t[0] = (((double)Sf(1,1) * Sf(2,2) - (double)Sf(1,2) * Sf(2,1)) * d);
+                    t[1] = (((double)Sf(0,2) * Sf(2,1) - (double)Sf(0,1) * Sf(2,2)) * d);
+                    t[2] = (((double)Sf(0,1) * Sf(1,2) - (double)Sf(0,2) * Sf(1,1)) * d);
 
-                    t[0] = (float)(((double)Sf(1,1) * Sf(2,2) - (double)Sf(1,2) * Sf(2,1)) * d);
-                    t[1] = (float)(((double)Sf(0,2) * Sf(2,1) - (double)Sf(0,1) * Sf(2,2)) * d);
-                    t[2] = (float)(((double)Sf(0,1) * Sf(1,2) - (double)Sf(0,2) * Sf(1,1)) * d);
-                           
-                    t[3] = (float)(((double)Sf(1,2) * Sf(2,0) - (double)Sf(1,0) * Sf(2,2)) * d);
-                    t[4] = (float)(((double)Sf(0,0) * Sf(2,2) - (double)Sf(0,2) * Sf(2,0)) * d);
-                    t[5] = (float)(((double)Sf(0,2) * Sf(1,0) - (double)Sf(0,0) * Sf(1,2)) * d);
-                           
-                    t[6] = (float)(((double)Sf(1,0) * Sf(2,1) - (double)Sf(1,1) * Sf(2,0)) * d);
-                    t[7] = (float)(((double)Sf(0,1) * Sf(2,0) - (double)Sf(0,0) * Sf(2,1)) * d);
-                    t[8] = (float)(((double)Sf(0,0) * Sf(1,1) - (double)Sf(0,1) * Sf(1,0)) * d);
+                    t[3] = (((double)Sf(1,2) * Sf(2,0) - (double)Sf(1,0) * Sf(2,2)) * d);
+                    t[4] = (((double)Sf(0,0) * Sf(2,2) - (double)Sf(0,2) * Sf(2,0)) * d);
+                    t[5] = (((double)Sf(0,2) * Sf(1,0) - (double)Sf(0,0) * Sf(1,2)) * d);
 
-                    Df(0,0) = t[0]; Df(0,1) = t[1]; Df(0,2) = t[2];
-                    Df(1,0) = t[3]; Df(1,1) = t[4]; Df(1,2) = t[5];
-                    Df(2,0) = t[6]; Df(2,1) = t[7]; Df(2,2) = t[8];
+                    t[6] = (((double)Sf(1,0) * Sf(2,1) - (double)Sf(1,1) * Sf(2,0)) * d);
+                    t[7] = (((double)Sf(0,1) * Sf(2,0) - (double)Sf(0,0) * Sf(2,1)) * d);
+                    t[8] = (((double)Sf(0,0) * Sf(1,1) - (double)Sf(0,1) * Sf(1,0)) * d);
+
+                    Df(0,0) = (float)t[0]; Df(0,1) = (float)t[1]; Df(0,2) = (float)t[2];
+                    Df(1,0) = (float)t[3]; Df(1,1) = (float)t[4]; Df(1,2) = (float)t[5];
+                    Df(2,0) = (float)t[6]; Df(2,1) = (float)t[7]; Df(2,2) = (float)t[8];
                 }
             }
             else
@@ -1077,18 +1110,18 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
                 double d = det3(Sd);
                 if( d != 0. )
                 {
-                    double t[9];
                     result = true;
                     d = 1./d;
+                    double t[9];
 
                     t[0] = (Sd(1,1) * Sd(2,2) - Sd(1,2) * Sd(2,1)) * d;
                     t[1] = (Sd(0,2) * Sd(2,1) - Sd(0,1) * Sd(2,2)) * d;
                     t[2] = (Sd(0,1) * Sd(1,2) - Sd(0,2) * Sd(1,1)) * d;
-                           
+
                     t[3] = (Sd(1,2) * Sd(2,0) - Sd(1,0) * Sd(2,2)) * d;
                     t[4] = (Sd(0,0) * Sd(2,2) - Sd(0,2) * Sd(2,0)) * d;
                     t[5] = (Sd(0,2) * Sd(1,0) - Sd(0,0) * Sd(1,2)) * d;
-                           
+
                     t[6] = (Sd(1,0) * Sd(2,1) - Sd(1,1) * Sd(2,0)) * d;
                     t[7] = (Sd(0,1) * Sd(2,0) - Sd(0,0) * Sd(2,1)) * d;
                     t[8] = (Sd(0,0) * Sd(1,1) - Sd(0,1) * Sd(1,0)) * d;
@@ -1127,12 +1160,12 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
         return result;
     }
 
-    int elem_size = CV_ELEM_SIZE(type);
+   int elem_size = CV_ELEM_SIZE(type);
     AutoBuffer<uchar> buf(n*n*elem_size);
     Mat src1(n, n, type, (uchar*)buf);
     src.copyTo(src1);
     setIdentity(dst);
-    
+
     if( method == DECOMP_LU && type == CV_32F )
         result = LU((float*)src1.data, src1.step, n, (float*)dst.data, dst.step, n) != 0;
     else if( method == DECOMP_LU && type == CV_64F )
@@ -1148,6 +1181,8 @@ double cv::invert( InputArray _src, OutputArray _dst, int method )
     return result;
 }
 
+
+
 /****************************************************************************************\
 *                              Solving a linear system                                   *
 \****************************************************************************************/
@@ -1171,7 +1206,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
     {
         _dst.create( src.cols, _src2.cols, src.type() );
         Mat dst = _dst.getMat();
-        
+
         #define bf(y) ((float*)(bdata + y*src2step))[0]
         #define bd(y) ((double*)(bdata + y*src2step))[0]
 
@@ -1253,7 +1288,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
                     double t[9];
 
                     d = 1./d;
-                    
+
                     t[0] = ((Sd(1,1) * Sd(2,2) - Sd(1,2) * Sd(2,1))*bd(0) +
                             (Sd(0,2) * Sd(2,1) - Sd(0,1) * Sd(2,2))*bd(1) +
                             (Sd(0,1) * Sd(1,2) - Sd(0,2) * Sd(1,1))*bd(2))*d;
@@ -1300,7 +1335,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
 
     if( method == DECOMP_QR )
         method = DECOMP_SVD;
-    
+
     int m = src.rows, m_ = m, n = src.cols, nb = _src2.cols;
     size_t esz = CV_ELEM_SIZE(type), bufsize = 0;
     size_t vstep = alignSize(n*esz, 16);
@@ -1310,10 +1345,10 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
     Mat src2 = _src2;
     _dst.create( src.cols, src2.cols, src.type() );
     Mat dst = _dst.getMat();
-    
+
     if( m < n )
         CV_Error(CV_StsBadArg, "The function can not solve under-determined linear systems" );
-    
+
     if( m == n )
         is_normal = false;
     else if( is_normal )
@@ -1322,7 +1357,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
         if( method == DECOMP_SVD )
             method = DECOMP_EIG;
     }
-    
+
     size_t asize = astep*(method == DECOMP_SVD || is_normal ? n : m);
     bufsize += asize + 32;
 
@@ -1331,7 +1366,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
 
     if( method == DECOMP_SVD || method == DECOMP_EIG )
         bufsize += n*5*esz + n*vstep + nb*sizeof(double) + 32;
-    
+
     buffer.allocate(bufsize);
     uchar* ptr = alignPtr((uchar*)buffer, 16);
 
@@ -1347,7 +1382,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
         transpose(src, a);
     }
     ptr += asize;
-    
+
     if( !is_normal )
     {
         if( method == DECOMP_LU || method == DECOMP_CHOLESKY )
@@ -1366,7 +1401,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
             src2 = tmp;
         }
     }
-    
+
     if( method == DECOMP_LU )
     {
         if( type == CV_32F )
@@ -1386,7 +1421,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
         ptr = alignPtr(ptr, 16);
         Mat v(n, n, type, ptr, vstep), w(n, 1, type, ptr + vstep*n), u;
         ptr += n*(vstep + esz);
-        
+
         if( method == DECOMP_EIG )
         {
             if( type == CV_32F )
@@ -1403,7 +1438,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
                 JacobiSVD(a.ptr<double>(), a.step, w.ptr<double>(), v.ptr<double>(), v.step, m_, n);
             u = a;
         }
-        
+
         if( type == CV_32F )
         {
             SVBkSb(m_, n, w.ptr<float>(), 0, u.ptr<float>(), u.step, true,
@@ -1418,7 +1453,7 @@ bool cv::solve( InputArray _src, InputArray _src2arg, OutputArray _dst, int meth
         }
         result = true;
     }
-    
+
     if( !result )
         dst = Scalar(0);
 
@@ -1443,7 +1478,7 @@ bool cv::eigen( InputArray _src, bool computeEvects, OutputArray _evals, OutputA
         _evects.create(n, n, type);
         v = _evects.getMat();
     }
-    
+
     size_t elemSize = src.elemSize(), astep = alignSize(n*elemSize, 16);
     AutoBuffer<uchar> buf(n*astep + n*5*elemSize + 32);
     uchar* ptr = alignPtr((uchar*)buf, 16);
@@ -1453,7 +1488,7 @@ bool cv::eigen( InputArray _src, bool computeEvects, OutputArray _evals, OutputA
     bool ok = type == CV_32F ?
         Jacobi(a.ptr<float>(), a.step, w.ptr<float>(), v.ptr<float>(), v.step, n, ptr) :
         Jacobi(a.ptr<double>(), a.step, w.ptr<double>(), v.ptr<double>(), v.step, n, ptr);
-    
+
     w.copyTo(_evals);
     return ok;
 }
@@ -1479,23 +1514,23 @@ static void _SVDcompute( InputArray _aarr, OutputArray _w,
     int type = src.type();
     bool compute_uv = _u.needed() || _vt.needed();
     bool full_uv = (flags & SVD::FULL_UV) != 0;
-    
+
     CV_Assert( type == CV_32F || type == CV_64F );
-    
+
     if( flags & SVD::NO_UV )
     {
         _u.release();
         _vt.release();
         compute_uv = full_uv = false;
     }
-    
+
     bool at = false;
     if( m < n )
     {
         std::swap(m, n);
         at = true;
     }
-    
+
     int urows = full_uv ? m : n;
     size_t esz = src.elemSize(), astep = alignSize(m*esz, 16), vstep = alignSize(n*esz, 16);
     AutoBuffer<uchar> _buf(urows*astep + n*vstep + n*esz + 32);
@@ -1503,18 +1538,18 @@ static void _SVDcompute( InputArray _aarr, OutputArray _w,
     Mat temp_a(n, m, type, buf, astep);
     Mat temp_w(n, 1, type, buf + urows*astep);
     Mat temp_u(urows, m, type, buf, astep), temp_v;
-    
+
     if( compute_uv )
         temp_v = Mat(n, n, type, alignPtr(buf + urows*astep + n*esz, 16), vstep);
-    
+
     if( urows > n )
         temp_u = Scalar::all(0);
-    
+
     if( !at )
         transpose(src, temp_a);
     else
         src.copyTo(temp_a);
-    
+
     if( type == CV_32F )
     {
         JacobiSVD(temp_a.ptr<float>(), temp_u.step, temp_w.ptr<float>(),
@@ -1539,9 +1574,9 @@ static void _SVDcompute( InputArray _aarr, OutputArray _w,
             temp_u.copyTo(_vt);
         }
     }
-}       
-    
-    
+}
+
+
 void SVD::compute( InputArray a, OutputArray w, OutputArray u, OutputArray vt, int flags )
 {
     _SVDcompute(a, w, u, vt, flags);
@@ -1551,20 +1586,20 @@ void SVD::compute( InputArray a, OutputArray w, int flags )
 {
     _SVDcompute(a, w, noArray(), noArray(), flags);
 }
-    
+
 void SVD::backSubst( InputArray _w, InputArray _u, InputArray _vt,
                      InputArray _rhs, OutputArray _dst )
 {
     Mat w = _w.getMat(), u = _u.getMat(), vt = _vt.getMat(), rhs = _rhs.getMat();
     int type = w.type(), esz = (int)w.elemSize();
     int m = u.rows, n = vt.cols, nb = rhs.data ? rhs.cols : m, nm = std::min(m, n);
-    size_t wstep = w.rows == 1 ? esz : w.cols == 1 ? (size_t)w.step : (size_t)w.step + esz;
+    size_t wstep = w.rows == 1 ? (size_t)esz : w.cols == 1 ? (size_t)w.step : (size_t)w.step + esz;
     AutoBuffer<uchar> buffer(nb*sizeof(double) + 16);
     CV_Assert( w.type() == u.type() && u.type() == vt.type() && u.data && vt.data && w.data );
     CV_Assert( u.cols >= nm && vt.rows >= nm &&
               (w.size() == Size(nm, 1) || w.size() == Size(1, nm) || w.size() == Size(vt.rows, u.cols)) );
     CV_Assert( rhs.data == 0 || (rhs.type() == type && rhs.rows == m) );
-    
+
     _dst.create( n, nb, type );
     Mat dst = _dst.getMat();
     if( type == CV_32F )
@@ -1579,7 +1614,7 @@ void SVD::backSubst( InputArray _w, InputArray _u, InputArray _vt,
         CV_Error( CV_StsUnsupportedFormat, "" );
 }
 
-    
+
 SVD& SVD::operator ()(InputArray a, int flags)
 {
     _SVDcompute(a, w, u, vt, flags);
@@ -1794,7 +1829,7 @@ cvSVBkSb( const CvArr* warr, const CvArr* uarr,
     }
     if( rhsarr )
         rhs = cv::cvarrToMat(rhsarr);
-    
+
     cv::SVD::backSubst(w, u, v, rhs, dst);
     CV_Assert( dst.data == dst0.data );
-}
+}
\ No newline at end of file
