@@ -222,7 +222,40 @@ static int countNonZero_(const T* src, int len )
 }
 
 static int countNonZero8u( const uchar* src, int len )
-{ return countNonZero_(src, len); }
+{
+    int i=0, nz = 0;
+#if CV_SSE2
+    if(USE_SSE2)//5x-6x
+    {
+        __m128i pattern = _mm_setzero_si128 ();
+        static uchar tab[256];
+        static volatile bool initialized = false;
+        if( !initialized )
+        {
+            // we compute inverse popcount table,
+            // since we pass (img[x] == 0) mask as index in the table.
+            for( int j = 0; j < 256; j++ )
+            {
+                int val = 0;
+                for( int mask = 1; mask < 256; mask += mask )
+                    val += (j & mask) == 0;
+                tab[j] = (uchar)val;
+            }
+            initialized = true;
+        }
+
+        for (; i<=len-16; i+=16)
+        {
+            __m128i r0 = _mm_loadu_si128((const __m128i*)(src+i));
+            int val = _mm_movemask_epi8(_mm_cmpeq_epi8(r0, pattern));
+            nz += tab[val & 255] + tab[val >> 8];
+        }
+    }
+#endif
+    for( ; i < len; i++ )
+        nz += src[i] != 0;
+    return nz;
+}
 
 static int countNonZero16u( const ushort* src, int len )
 { return countNonZero_(src, len); }
@@ -966,25 +999,22 @@ static int normHamming(const uchar* a, int n)
 {
     int i = 0, result = 0;
 #if CV_NEON
-    if (CPU_HAS_NEON_FEATURE)
-    {
-        uint32x4_t bits = vmovq_n_u32(0);
-        for (; i <= n - 16; i += 16) {
-            uint8x16_t A_vec = vld1q_u8 (a + i);
-            uint8x16_t bitsSet = vcntq_u8 (A_vec);
-            uint16x8_t bitSet8 = vpaddlq_u8 (bitsSet);
-            uint32x4_t bitSet4 = vpaddlq_u16 (bitSet8);
-            bits = vaddq_u32(bits, bitSet4);
-        }
-        uint64x2_t bitSet2 = vpaddlq_u32 (bits);
-        result = vgetq_lane_s32 (vreinterpretq_s32_u64(bitSet2),0);
-        result += vgetq_lane_s32 (vreinterpretq_s32_u64(bitSet2),2);
+    uint32x4_t bits = vmovq_n_u32(0);
+    for (; i <= n - 16; i += 16) {
+        uint8x16_t A_vec = vld1q_u8 (a + i);
+        uint8x16_t bitsSet = vcntq_u8 (A_vec);
+        uint16x8_t bitSet8 = vpaddlq_u8 (bitsSet);
+        uint32x4_t bitSet4 = vpaddlq_u16 (bitSet8);
+        bits = vaddq_u32(bits, bitSet4);
     }
-    else
-#endif
-        for( ; i <= n - 4; i += 4 )
+    uint64x2_t bitSet2 = vpaddlq_u32 (bits);
+    result = vgetq_lane_s32 (vreinterpretq_s32_u64(bitSet2),0);
+    result += vgetq_lane_s32 (vreinterpretq_s32_u64(bitSet2),2);
+#else
+    for( ; i <= n - 4; i += 4 )
             result += popCountTable[a[i]] + popCountTable[a[i+1]] +
             popCountTable[a[i+2]] + popCountTable[a[i+3]];
+#endif
     for( ; i < n; i++ )
         result += popCountTable[a[i]];
     return result;
@@ -994,27 +1024,24 @@ int normHamming(const uchar* a, const uchar* b, int n)
 {
     int i = 0, result = 0;
 #if CV_NEON
-    if (CPU_HAS_NEON_FEATURE)
-    {
-        uint32x4_t bits = vmovq_n_u32(0);
-        for (; i <= n - 16; i += 16) {
-            uint8x16_t A_vec = vld1q_u8 (a + i);
-            uint8x16_t B_vec = vld1q_u8 (b + i);
-            uint8x16_t AxorB = veorq_u8 (A_vec, B_vec);
-            uint8x16_t bitsSet = vcntq_u8 (AxorB);
-            uint16x8_t bitSet8 = vpaddlq_u8 (bitsSet);
-            uint32x4_t bitSet4 = vpaddlq_u16 (bitSet8);
-            bits = vaddq_u32(bits, bitSet4);
-        }
-        uint64x2_t bitSet2 = vpaddlq_u32 (bits);
-        result = vgetq_lane_s32 (vreinterpretq_s32_u64(bitSet2),0);
-        result += vgetq_lane_s32 (vreinterpretq_s32_u64(bitSet2),2);
+    uint32x4_t bits = vmovq_n_u32(0);
+    for (; i <= n - 16; i += 16) {
+        uint8x16_t A_vec = vld1q_u8 (a + i);
+        uint8x16_t B_vec = vld1q_u8 (b + i);
+        uint8x16_t AxorB = veorq_u8 (A_vec, B_vec);
+        uint8x16_t bitsSet = vcntq_u8 (AxorB);
+        uint16x8_t bitSet8 = vpaddlq_u8 (bitsSet);
+        uint32x4_t bitSet4 = vpaddlq_u16 (bitSet8);
+        bits = vaddq_u32(bits, bitSet4);
     }
-    else
+    uint64x2_t bitSet2 = vpaddlq_u32 (bits);
+    result = vgetq_lane_s32 (vreinterpretq_s32_u64(bitSet2),0);
+    result += vgetq_lane_s32 (vreinterpretq_s32_u64(bitSet2),2);
+#else
+    for( ; i <= n - 4; i += 4 )
+        result += popCountTable[a[i] ^ b[i]] + popCountTable[a[i+1] ^ b[i+1]] +
+                popCountTable[a[i+2] ^ b[i+2]] + popCountTable[a[i+3] ^ b[i+3]];
 #endif
-        for( ; i <= n - 4; i += 4 )
-            result += popCountTable[a[i] ^ b[i]] + popCountTable[a[i+1] ^ b[i+1]] +
-                    popCountTable[a[i+2] ^ b[i+2]] + popCountTable[a[i+3] ^ b[i+3]];
     for( ; i < n; i++ )
         result += popCountTable[a[i] ^ b[i]];
     return result;
@@ -1693,7 +1720,7 @@ typedef void (*BatchDistFunc)(const uchar* src1, const uchar* src2, size_t step2
                               int nvecs, int len, uchar* dist, const uchar* mask);
 
 
-struct BatchDistInvoker
+struct BatchDistInvoker : public ParallelLoopBody
 {
     BatchDistInvoker( const Mat& _src1, const Mat& _src2,
                       Mat& _dist, Mat& _nidx, int _K,
@@ -1710,12 +1737,12 @@ struct BatchDistInvoker
         func = _func;
     }
 
-    void operator()(const BlockedRange& range) const
+    void operator()(const Range& range) const
     {
         AutoBuffer<int> buf(src2->rows);
         int* bufptr = buf;
 
-        for( int i = range.begin(); i < range.end(); i++ )
+        for( int i = range.start; i < range.end; i++ )
         {
             func(src1->ptr(i), src2->ptr(), src2->step, src2->rows, src2->cols,
                  K > 0 ? (uchar*)bufptr : dist->ptr(i), mask->data ? mask->ptr(i) : 0);
@@ -1812,7 +1839,7 @@ void cv::batchDistance( InputArray _src1, InputArray _src2,
                 int d = tdist.at<int>(i), d0 = dist.at<int>(idx);
                 if( d < d0 )
                 {
-                    dist.at<int>(idx) = d0;
+                    dist.at<int>(idx) = d;
                     nidx.at<int>(idx) = i + update;
                 }
             }
@@ -1825,7 +1852,7 @@ void cv::batchDistance( InputArray _src1, InputArray _src2,
                 float d = tdist.at<float>(i), d0 = dist.at<float>(idx);
                 if( d < d0 )
                 {
-                    dist.at<float>(idx) = d0;
+                    dist.at<float>(idx) = d;
                     nidx.at<int>(idx) = i + update;
                 }
             }
@@ -1866,8 +1893,30 @@ void cv::batchDistance( InputArray _src1, InputArray _src2,
                   ("The combination of type=%d, dtype=%d and normType=%d is not supported",
                    type, dtype, normType));
 
-    parallel_for(BlockedRange(0, src1.rows),
-                 BatchDistInvoker(src1, src2, dist, nidx, K, mask, update, func));
+    parallel_for_(Range(0, src1.rows),
+                  BatchDistInvoker(src1, src2, dist, nidx, K, mask, update, func));
+}
+
+
+void cv::findNonZero( InputArray _src, OutputArray _idx )
+{
+    Mat src = _src.getMat();
+    CV_Assert( src.type() == CV_8UC1 );
+    int n = countNonZero(src);
+    if( _idx.kind() == _InputArray::MAT && !_idx.getMatRef().isContinuous() )
+        _idx.release();
+    _idx.create(n, 1, CV_32SC2);
+    Mat idx = _idx.getMat();
+    CV_Assert(idx.isContinuous());
+    Point* idx_ptr = (Point*)idx.data;
+
+    for( int i = 0; i < src.rows; i++ )
+    {
+        const uchar* bin_ptr = src.ptr(i);
+        for( int j = 0; j < src.cols; j++ )
+            if( bin_ptr[j] )
+                *idx_ptr++ = Point(j, i);
+    }
 }
 
 
