@@ -1522,4 +1522,45 @@ protected:
 
 TEST(Core_ArithmMask, uninitialized) { CV_ArithmMaskTest test; test.safe_run(); }
 
+TEST(Multiply, FloatingPointRounding)
+{
+    cv::Mat src(1, 1, CV_8UC1, cv::Scalar::all(110)), dst;
+    cv::Scalar s(147.286359696927, 1, 1 ,1);
 
+    cv::multiply(src, s, dst, 1, CV_16U);
+    // with CV_32F this produce result 16202
+    ASSERT_EQ(dst.at<ushort>(0,0), 16201);
+}
+
+TEST(Core_Add, AddToColumnWhen3Rows)
+{
+    cv::Mat m1 = (cv::Mat_<double>(3, 2) << 1, 2, 3, 4, 5, 6);
+    m1.col(1) += 10;
+
+    cv::Mat m2 = (cv::Mat_<double>(3, 2) << 1, 12, 3, 14, 5, 16);
+
+    ASSERT_EQ(0, countNonZero(m1 - m2));
+}
+
+TEST(Core_Add, AddToColumnWhen4Rows)
+{
+    cv::Mat m1 = (cv::Mat_<double>(4, 2) << 1, 2, 3, 4, 5, 6, 7, 8);
+    m1.col(1) += 10;
+
+    cv::Mat m2 = (cv::Mat_<double>(4, 2) << 1, 12, 3, 14, 5, 16, 7, 18);
+
+    ASSERT_EQ(0, countNonZero(m1 - m2));
+}
+
+TEST(Core_round, CvRound)
+{
+    ASSERT_EQ(2, cvRound(2.0));
+    ASSERT_EQ(2, cvRound(2.1));
+    ASSERT_EQ(-2, cvRound(-2.1));
+    ASSERT_EQ(3, cvRound(2.8));
+    ASSERT_EQ(-3, cvRound(-2.8));
+    ASSERT_EQ(2, cvRound(2.5));
+    ASSERT_EQ(4, cvRound(3.5));
+    ASSERT_EQ(-2, cvRound(-2.5));
+    ASSERT_EQ(-4, cvRound(-3.5));
+}
\ No newline at end of file
