@@ -1,5 +1,7 @@
 #include "perf_precomp.hpp"
 
+#if BUILD_WITH_VIDEO_OUTPUT_SUPPORT
+
 using namespace std;
 using namespace cv;
 using namespace perf;
@@ -21,9 +23,12 @@ PERF_TEST_P(VideoWriter_Writing, WriteFrame,
   string filename = getDataPath(get<0>(GetParam()));
   bool isColor = get<1>(GetParam());
 
-  VideoWriter writer("perf_writer.avi", CV_FOURCC('X', 'V', 'I', 'D'), 25, cv::Size(640, 480), isColor);
+  VideoWriter writer(cv::tempfile(".avi"), CV_FOURCC('X', 'V', 'I', 'D'), 25, cv::Size(640, 480), isColor);
 
   TEST_CYCLE() { Mat image = imread(filename, 1); writer << image; }
 
-  SANITY_CHECK(writer.isOpened());
+  bool dummy = writer.isOpened();
+  SANITY_CHECK(dummy);
 }
+
+#endif // BUILD_WITH_VIDEO_OUTPUT_SUPPORT
\ No newline at end of file
