@@ -49,6 +49,10 @@
 #pragma warning( disable: 4244 4510 4512 4610 )
 #endif
 
+#ifdef __GNUC__
+#  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -140,6 +144,18 @@ extern "C" {
 #define AV_NOPTS_VALUE_ ((int64_t)AV_NOPTS_VALUE)
 #endif
 
+#ifndef AVERROR_EOF
+#define AVERROR_EOF (-MKTAG( 'E','O','F',' '))
+#endif
+
+#if LIBAVCODEC_BUILD >= CALC_FFMPEG_VERSION(54,25,0)
+#  define CV_CODEC_ID AVCodecID
+#  define CV_CODEC(name) AV_##name
+#else
+#  define CV_CODEC_ID CodecID
+#  define CV_CODEC(name) name
+#endif
+
 static int get_number_of_cpus(void)
 {
 #if LIBAVFORMAT_BUILD < CALC_FFMPEG_VERSION(52, 111, 0)
@@ -324,28 +340,187 @@ void CvCapture_FFMPEG::close()
 #define AVSEEK_FLAG_ANY 1
 #endif
 
-static void icvInitFFMPEG_internal()
+class ImplMutex
 {
-    static volatile bool initialized = false;
-    if( !initialized )
+public:
+	ImplMutex() { init(); }
+	~ImplMutex() { destroy(); }
+	
+    void init();
+    void destroy();
+
+    void lock();
+    bool trylock();
+    void unlock();
+
+    struct Impl;
+protected:
+    Impl* impl;
+
+private:
+    ImplMutex(const ImplMutex&);
+    ImplMutex& operator = (const ImplMutex& m);
+};
+
+#if defined WIN32 || defined _WIN32 || defined WINCE
+
+struct ImplMutex::Impl
+{
+    void init() { InitializeCriticalSection(&cs); refcount = 1; }
+    void destroy() { DeleteCriticalSection(&cs); }
+
+    void lock() { EnterCriticalSection(&cs); }
+    bool trylock() { return TryEnterCriticalSection(&cs) != 0; }
+    void unlock() { LeaveCriticalSection(&cs); }
+
+    CRITICAL_SECTION cs;
+    int refcount;
+};
+
+#ifndef __GNUC__
+static int _interlockedExchangeAdd(int* addr, int delta)
+{
+#if defined _MSC_VER && _MSC_VER >= 1500
+    return (int)_InterlockedExchangeAdd((long volatile*)addr, delta);
+#else
+    return (int)InterlockedExchangeAdd((long volatile*)addr, delta);
+#endif
+}
+#endif // __GNUC__
+
+#elif defined __APPLE__
+
+#include <libkern/OSAtomic.h>
+
+struct ImplMutex::Impl
+{
+    void init() { sl = OS_SPINLOCK_INIT; refcount = 1; }
+    void destroy() { }
+
+    void lock() { OSSpinLockLock(&sl); }
+    bool trylock() { return OSSpinLockTry(&sl); }
+    void unlock() { OSSpinLockUnlock(&sl); }
+
+    OSSpinLock sl;
+    int refcount;
+};
+
+#elif defined __linux__ && !defined ANDROID
+
+struct ImplMutex::Impl
+{
+    void init() { pthread_spin_init(&sl, 0); refcount = 1; }
+    void destroy() { pthread_spin_destroy(&sl); }
+
+    void lock() { pthread_spin_lock(&sl); }
+    bool trylock() { return pthread_spin_trylock(&sl) == 0; }
+    void unlock() { pthread_spin_unlock(&sl); }
+
+    pthread_spinlock_t sl;
+    int refcount;
+};
+
+#else
+
+struct ImplMutex::Impl
+{
+    void init() { pthread_mutex_init(&sl, 0); refcount = 1; }
+    void destroy() { pthread_mutex_destroy(&sl); }
+
+    void lock() { pthread_mutex_lock(&sl); }
+    bool trylock() { return pthread_mutex_trylock(&sl) == 0; }
+    void unlock() { pthread_mutex_unlock(&sl); }
+
+    pthread_mutex_t sl;
+    int refcount;
+};
+
+#endif
+
+void ImplMutex::init()
+{
+	impl = (Impl*)malloc(sizeof(Impl));
+	impl->init();
+}
+void ImplMutex::destroy() 
+{
+	impl->destroy();
+	free(impl);
+	impl = NULL;
+}
+void ImplMutex::lock() { impl->lock(); }
+void ImplMutex::unlock() { impl->unlock(); }
+bool ImplMutex::trylock() { return impl->trylock(); }
+
+static int LockCallBack(void **mutex, AVLockOp op)
+{
+    ImplMutex* localMutex = reinterpret_cast<ImplMutex*>(*mutex);
+    switch (op)
     {
+        case AV_LOCK_CREATE:
+            localMutex = reinterpret_cast<ImplMutex*>(malloc(sizeof(ImplMutex)));
+            localMutex->init();
+            *mutex = localMutex;
+            if (!*mutex)
+                return 1;
+        break;
+
+        case AV_LOCK_OBTAIN:
+            localMutex->lock();
+        break;
+
+        case AV_LOCK_RELEASE:
+            localMutex->unlock();
+        break;
+
+        case AV_LOCK_DESTROY:
+            localMutex->destroy();
+            free(localMutex);
+            localMutex = NULL;
+        break;
+    }
+    return 0;
+}
+
+static ImplMutex _mutex;
+static bool _initialized = false;
+
+class InternalFFMpegRegister
+{
+public:
+    InternalFFMpegRegister()
+    {
+        _mutex.lock();
+        if (!_initialized)
+        {
     #if LIBAVFORMAT_BUILD >= CALC_FFMPEG_VERSION(53, 13, 0)
-        avformat_network_init();
+            avformat_network_init();
     #endif
 
-        /* register all codecs, demux and protocols */
-        av_register_all();
+            /* register all codecs, demux and protocols */
+            av_register_all();
 
-        av_log_set_level(AV_LOG_ERROR);
+            /* register a callback function for synchronization */
+            av_lockmgr_register(&LockCallBack);
 
-        initialized = true;
+            av_log_set_level(AV_LOG_ERROR);
+
+            _initialized = true;
+        }
+        _mutex.unlock();
     }
-}
+
+    ~InternalFFMpegRegister()
+    {
+        _initialized = false;
+        av_lockmgr_register(NULL);
+    }
+};
+
+static InternalFFMpegRegister _init;
 
 bool CvCapture_FFMPEG::open( const char* _filename )
 {
-    icvInitFFMPEG_internal();
-
     unsigned i;
     bool valid = false;
 
@@ -357,7 +532,8 @@ bool CvCapture_FFMPEG::open( const char* _filename )
     int err = av_open_input_file(&ic, _filename, NULL, 0, NULL);
 #endif
 
-    if (err < 0) {
+    if (err < 0)
+    {
         CV_WARN("Error opening file");
         goto exit_func;
     }
@@ -367,7 +543,8 @@ bool CvCapture_FFMPEG::open( const char* _filename )
 #else
     av_find_stream_info(ic);
 #endif
-    if (err < 0) {
+    if (err < 0)
+    {
         CV_WARN("Could not find codec parameters");
         goto exit_func;
     }
@@ -379,17 +556,22 @@ bool CvCapture_FFMPEG::open( const char* _filename )
         AVCodecContext *enc = &ic->streams[i]->codec;
 #endif
 
-#ifdef FF_API_THREAD_INIT
-        avcodec_thread_init(enc, get_number_of_cpus());
-#else
+//#ifdef FF_API_THREAD_INIT
+//        avcodec_thread_init(enc, get_number_of_cpus());
+//#else
         enc->thread_count = get_number_of_cpus();
-#endif
+//#endif
 
 #if LIBAVFORMAT_BUILD < CALC_FFMPEG_VERSION(53, 2, 0)
 #define AVMEDIA_TYPE_VIDEO CODEC_TYPE_VIDEO
 #endif
 
-        if( AVMEDIA_TYPE_VIDEO == enc->codec_type && video_stream < 0) {
+        if( AVMEDIA_TYPE_VIDEO == enc->codec_type && video_stream < 0)
+        {
+            // backup encoder' width/height
+            int enc_width = enc->width;
+            int enc_height = enc->height;
+
             AVCodec *codec = avcodec_find_decoder(enc->codec_id);
             if (!codec ||
 #if LIBAVCODEC_VERSION_INT >= ((53<<16)+(8<<8)+0)
@@ -397,7 +579,12 @@ bool CvCapture_FFMPEG::open( const char* _filename )
 #else
                 avcodec_open(enc, codec)
 #endif
-                < 0) goto exit_func;
+                < 0)
+                goto exit_func;
+
+            // checking width/height (since decoder can sometimes alter it, eg. vp6f)
+            if (enc_width && (enc->width != enc_width)) { enc->width = enc_width; }
+            if (enc_height && (enc->height != enc_height)) { enc->height = enc_height; }
 
             video_stream = i;
             video_st = ic->streams[i];
@@ -842,7 +1029,7 @@ static const char * icvFFMPEGErrStr(int err)
 
 /* function internal to FFMPEG (libavformat/riff.c) to lookup codec id by fourcc tag*/
 extern "C" {
-    enum CodecID codec_get_bmp_id(unsigned int tag);
+    enum CV_CODEC_ID codec_get_bmp_id(unsigned int tag);
 }
 
 void CvVideoWriter_FFMPEG::init()
@@ -894,7 +1081,7 @@ static AVFrame * icv_alloc_picture_FFMPEG(int pix_fmt, int width, int height, bo
 
 /* add a video output stream to the container */
 static AVStream *icv_add_video_stream_FFMPEG(AVFormatContext *oc,
-                                             CodecID codec_id,
+                                             CV_CODEC_ID codec_id,
                                              int w, int h, int bitrate,
                                              double fps, int pixel_format)
 {
@@ -926,7 +1113,7 @@ static AVStream *icv_add_video_stream_FFMPEG(AVFormatContext *oc,
     c->codec_id = oc->oformat->video_codec;
 #endif
 
-    if(codec_id != CODEC_ID_NONE){
+    if(codec_id != CV_CODEC(CODEC_ID_NONE)){
         c->codec_id = codec_id;
     }
 
@@ -987,10 +1174,10 @@ static AVStream *icv_add_video_stream_FFMPEG(AVFormatContext *oc,
     c->gop_size = 12; /* emit one intra frame every twelve frames at most */
     c->pix_fmt = (PixelFormat) pixel_format;
 
-    if (c->codec_id == CODEC_ID_MPEG2VIDEO) {
+    if (c->codec_id == CV_CODEC(CODEC_ID_MPEG2VIDEO)) {
         c->max_b_frames = 2;
     }
-    if (c->codec_id == CODEC_ID_MPEG1VIDEO || c->codec_id == CODEC_ID_MSMPEG4V3){
+    if (c->codec_id == CV_CODEC(CODEC_ID_MPEG1VIDEO) || c->codec_id == CV_CODEC(CODEC_ID_MSMPEG4V3)){
         /* needed to avoid using macroblocks in which some coeffs overflow
            this doesnt happen with normal video, it just happens here as the
            motion of the chroma plane doesnt match the luma plane */
@@ -1084,7 +1271,7 @@ bool CvVideoWriter_FFMPEG::writeFrame( const unsigned char* data, int step, int
 
 #if LIBAVFORMAT_BUILD < 5231
     // It is not needed in the latest versions of the ffmpeg
-    if( c->codec_id == CODEC_ID_RAWVIDEO && origin != 1 )
+    if( c->codec_id == CV_CODEC(CODEC_ID_RAWVIDEO) && origin != 1 )
     {
         if( !temp_image.data )
         {
@@ -1271,9 +1458,7 @@ void CvVideoWriter_FFMPEG::close()
 bool CvVideoWriter_FFMPEG::open( const char * filename, int fourcc,
                                  double fps, int width, int height, bool is_color )
 {
-    icvInitFFMPEG_internal();
-
-    CodecID codec_id = CODEC_ID_NONE;
+    CV_CODEC_ID codec_id = CV_CODEC(CODEC_ID_NONE);
     int err, codec_pix_fmt;
     double bitrate_scale = 1;
 
@@ -1314,11 +1499,11 @@ bool CvVideoWriter_FFMPEG::open( const char * filename, int fourcc,
 
     /* Lookup codec_id for given fourcc */
 #if LIBAVCODEC_VERSION_INT<((51<<16)+(49<<8)+0)
-    if( (codec_id = codec_get_bmp_id( fourcc )) == CODEC_ID_NONE )
+    if( (codec_id = codec_get_bmp_id( fourcc )) == CV_CODEC(CODEC_ID_NONE) )
         return false;
 #else
     const struct AVCodecTag * tags[] = { codec_bmp_tags, NULL};
-    if( (codec_id = av_codec_get_id(tags, fourcc)) == CODEC_ID_NONE )
+    if( (codec_id = av_codec_get_id(tags, fourcc)) == CV_CODEC(CODEC_ID_NONE) )
         return false;
 #endif
 
@@ -1340,20 +1525,20 @@ bool CvVideoWriter_FFMPEG::open( const char * filename, int fourcc,
     // set a few optimal pixel formats for lossless codecs of interest..
     switch (codec_id) {
 #if LIBAVCODEC_VERSION_INT>((50<<16)+(1<<8)+0)
-    case CODEC_ID_JPEGLS:
+    case CV_CODEC(CODEC_ID_JPEGLS):
         // BGR24 or GRAY8 depending on is_color...
         codec_pix_fmt = input_pix_fmt;
         break;
 #endif
-    case CODEC_ID_HUFFYUV:
+    case CV_CODEC(CODEC_ID_HUFFYUV):
         codec_pix_fmt = PIX_FMT_YUV422P;
         break;
-    case CODEC_ID_MJPEG:
-    case CODEC_ID_LJPEG:
+    case CV_CODEC(CODEC_ID_MJPEG):
+    case CV_CODEC(CODEC_ID_LJPEG):
         codec_pix_fmt = PIX_FMT_YUVJ420P;
         bitrate_scale = 3;
         break;
-    case CODEC_ID_RAWVIDEO:
+    case CV_CODEC(CODEC_ID_RAWVIDEO):
         codec_pix_fmt = input_pix_fmt == PIX_FMT_GRAY8 ||
                         input_pix_fmt == PIX_FMT_GRAY16LE ||
                         input_pix_fmt == PIX_FMT_GRAY16BE ? input_pix_fmt : PIX_FMT_YUV420P;
@@ -1491,6 +1676,7 @@ bool CvVideoWriter_FFMPEG::open( const char * filename, int fourcc,
     frame_width = width;
     frame_height = height;
     ok = true;
+
     return true;
 }
 
@@ -1502,6 +1688,7 @@ CvCapture_FFMPEG* cvCreateFileCapture_FFMPEG( const char* filename )
     capture->init();
     if( capture->open( filename ))
         return capture;
+
     capture->close();
     free(capture);
     return 0;
@@ -1550,7 +1737,6 @@ CvVideoWriter_FFMPEG* cvCreateVideoWriter_FFMPEG( const char* filename, int four
     return 0;
 }
 
-
 void cvReleaseVideoWriter_FFMPEG( CvVideoWriter_FFMPEG** writer )
 {
     if( writer && *writer )
@@ -1569,3 +1755,518 @@ int cvWriteFrame_FFMPEG( CvVideoWriter_FFMPEG* writer,
     return writer->writeFrame(data, step, width, height, cn, origin);
 }
 
+
+
+/*
+ * For CUDA encoder
+ */
+
+struct OutputMediaStream_FFMPEG
+{
+    bool open(const char* fileName, int width, int height, double fps);
+    void close();
+
+    void write(unsigned char* data, int size, int keyFrame);
+
+    // add a video output stream to the container
+    static AVStream* addVideoStream(AVFormatContext *oc, CV_CODEC_ID codec_id, int w, int h, int bitrate, double fps, PixelFormat pixel_format);
+
+    AVOutputFormat* fmt_;
+    AVFormatContext* oc_;
+    AVStream* video_st_;
+};
+
+void OutputMediaStream_FFMPEG::close()
+{
+    // no more frame to compress. The codec has a latency of a few
+    // frames if using B frames, so we get the last frames by
+    // passing the same picture again
+
+    // TODO -- do we need to account for latency here?
+
+    if (oc_)
+    {
+        // write the trailer, if any
+        av_write_trailer(oc_);
+
+        // free the streams
+        for (unsigned int i = 0; i < oc_->nb_streams; ++i)
+        {
+            av_freep(&oc_->streams[i]->codec);
+            av_freep(&oc_->streams[i]);
+        }
+
+        if (!(fmt_->flags & AVFMT_NOFILE) && oc_->pb)
+        {
+            // close the output file
+
+            #if LIBAVCODEC_VERSION_INT < ((52<<16)+(123<<8)+0)
+                #if LIBAVCODEC_VERSION_INT >= ((51<<16)+(49<<8)+0)
+                    url_fclose(oc_->pb);
+                #else
+                    url_fclose(&oc_->pb);
+                #endif
+            #else
+                avio_close(oc_->pb);
+            #endif
+        }
+
+        // free the stream
+        av_free(oc_);
+    }
+}
+
+AVStream* OutputMediaStream_FFMPEG::addVideoStream(AVFormatContext *oc, CV_CODEC_ID codec_id, int w, int h, int bitrate, double fps, PixelFormat pixel_format)
+{
+    #if LIBAVFORMAT_BUILD >= CALC_FFMPEG_VERSION(53, 10, 0)
+        AVStream* st = avformat_new_stream(oc, 0);
+    #else
+        AVStream* st = av_new_stream(oc, 0);
+    #endif
+    if (!st)
+        return 0;
+
+    #if LIBAVFORMAT_BUILD > 4628
+        AVCodecContext* c = st->codec;
+    #else
+        AVCodecContext* c = &(st->codec);
+    #endif
+
+    c->codec_id = codec_id;
+    c->codec_type = AVMEDIA_TYPE_VIDEO;
+
+    // put sample parameters
+    unsigned long long lbit_rate = static_cast<unsigned long long>(bitrate);
+    lbit_rate += (bitrate / 4);
+    lbit_rate = std::min(lbit_rate, static_cast<unsigned long long>(std::numeric_limits<int>::max()));
+    c->bit_rate = bitrate;
+
+    // took advice from
+    // http://ffmpeg-users.933282.n4.nabble.com/warning-clipping-1-dct-coefficients-to-127-127-td934297.html
+    c->qmin = 3;
+
+    // resolution must be a multiple of two
+    c->width = w;
+    c->height = h;
+
+    AVCodec* codec = avcodec_find_encoder(c->codec_id);
+
+    // time base: this is the fundamental unit of time (in seconds) in terms
+    // of which frame timestamps are represented. for fixed-fps content,
+    // timebase should be 1/framerate and timestamp increments should be
+    // identically 1
+
+    int frame_rate = static_cast<int>(fps+0.5);
+    int frame_rate_base = 1;
+    while (fabs(static_cast<double>(frame_rate)/frame_rate_base) - fps > 0.001)
+    {
+        frame_rate_base *= 10;
+        frame_rate = static_cast<int>(fps*frame_rate_base + 0.5);
+    }
+    c->time_base.den = frame_rate;
+    c->time_base.num = frame_rate_base;
+
+    #if LIBAVFORMAT_BUILD > 4752
+        // adjust time base for supported framerates
+        if (codec && codec->supported_framerates)
+        {
+            AVRational req = {frame_rate, frame_rate_base};
+            const AVRational* best = NULL;
+            AVRational best_error = {INT_MAX, 1};
+
+            for (const AVRational* p = codec->supported_framerates; p->den!=0; ++p)
+            {
+                AVRational error = av_sub_q(req, *p);
+
+                if (error.num < 0)
+                    error.num *= -1;
+
+                if (av_cmp_q(error, best_error) < 0)
+                {
+                    best_error= error;
+                    best= p;
+                }
+            }
+
+            c->time_base.den= best->num;
+            c->time_base.num= best->den;
+        }
+    #endif
+
+    c->gop_size = 12; // emit one intra frame every twelve frames at most
+    c->pix_fmt = pixel_format;
+
+    if (c->codec_id == CV_CODEC(CODEC_ID_MPEG2VIDEO))
+        c->max_b_frames = 2;
+
+    if (c->codec_id == CV_CODEC(CODEC_ID_MPEG1VIDEO) || c->codec_id == CV_CODEC(CODEC_ID_MSMPEG4V3))
+    {
+        // needed to avoid using macroblocks in which some coeffs overflow
+        // this doesnt happen with normal video, it just happens here as the
+        // motion of the chroma plane doesnt match the luma plane
+
+        // avoid FFMPEG warning 'clipping 1 dct coefficients...'
+
+        c->mb_decision = 2;
+    }
+
+    #if LIBAVCODEC_VERSION_INT > 0x000409
+        // some formats want stream headers to be seperate
+        if (oc->oformat->flags & AVFMT_GLOBALHEADER)
+        {
+            c->flags |= CODEC_FLAG_GLOBAL_HEADER;
+        }
+    #endif
+
+    return st;
+}
+
+bool OutputMediaStream_FFMPEG::open(const char* fileName, int width, int height, double fps)
+{
+    fmt_ = 0;
+    oc_ = 0;
+    video_st_ = 0;
+
+    // auto detect the output format from the name and fourcc code
+    #if LIBAVFORMAT_BUILD >= CALC_FFMPEG_VERSION(53, 2, 0)
+        fmt_ = av_guess_format(NULL, fileName, NULL);
+    #else
+        fmt_ = guess_format(NULL, fileName, NULL);
+    #endif
+    if (!fmt_)
+        return false;
+
+    CV_CODEC_ID codec_id = CV_CODEC(CODEC_ID_H264);
+
+    // alloc memory for context
+    #if LIBAVFORMAT_BUILD >= CALC_FFMPEG_VERSION(53, 2, 0)
+        oc_ = avformat_alloc_context();
+    #else
+        oc_ = av_alloc_format_context();
+    #endif
+    if (!oc_)
+        return false;
+
+    // set some options
+    oc_->oformat = fmt_;
+    snprintf(oc_->filename, sizeof(oc_->filename), "%s", fileName);
+
+    oc_->max_delay = (int)(0.7 * AV_TIME_BASE); // This reduces buffer underrun warnings with MPEG
+
+    // set a few optimal pixel formats for lossless codecs of interest..
+    PixelFormat codec_pix_fmt = PIX_FMT_YUV420P;
+    int bitrate_scale = 64;
+
+    // TODO -- safe to ignore output audio stream?
+    video_st_ = addVideoStream(oc_, codec_id, width, height, width * height * bitrate_scale, fps, codec_pix_fmt);
+    if (!video_st_)
+        return false;
+
+    // set the output parameters (must be done even if no parameters)
+    #if LIBAVFORMAT_BUILD < CALC_FFMPEG_VERSION(53, 2, 0)
+        if (av_set_parameters(oc_, NULL) < 0)
+            return false;
+    #endif
+
+    // now that all the parameters are set, we can open the audio and
+    // video codecs and allocate the necessary encode buffers
+
+    #if LIBAVFORMAT_BUILD > 4628
+        AVCodecContext* c = (video_st_->codec);
+    #else
+        AVCodecContext* c = &(video_st_->codec);
+    #endif
+
+    c->codec_tag = MKTAG('H', '2', '6', '4');
+    c->bit_rate_tolerance = c->bit_rate;
+
+    // open the output file, if needed
+    if (!(fmt_->flags & AVFMT_NOFILE))
+    {
+        #if LIBAVFORMAT_BUILD < CALC_FFMPEG_VERSION(53, 2, 0)
+            int err = url_fopen(&oc_->pb, fileName, URL_WRONLY);
+        #else
+            int err = avio_open(&oc_->pb, fileName, AVIO_FLAG_WRITE);
+        #endif
+
+        if (err != 0)
+            return false;
+    }
+
+    // write the stream header, if any
+    #if LIBAVFORMAT_BUILD < CALC_FFMPEG_VERSION(53, 2, 0)
+        av_write_header(oc_);
+    #else
+        avformat_write_header(oc_, NULL);
+    #endif
+
+    return true;
+}
+
+void OutputMediaStream_FFMPEG::write(unsigned char* data, int size, int keyFrame)
+{
+    // if zero size, it means the image was buffered
+    if (size > 0)
+    {
+        AVPacket pkt;
+        av_init_packet(&pkt);
+
+        if (keyFrame)
+            pkt.flags |= PKT_FLAG_KEY;
+
+        pkt.stream_index = video_st_->index;
+        pkt.data = data;
+        pkt.size = size;
+
+        // write the compressed frame in the media file
+        av_write_frame(oc_, &pkt);
+    }
+}
+
+struct OutputMediaStream_FFMPEG* create_OutputMediaStream_FFMPEG(const char* fileName, int width, int height, double fps)
+{
+    OutputMediaStream_FFMPEG* stream = (OutputMediaStream_FFMPEG*) malloc(sizeof(OutputMediaStream_FFMPEG));
+
+    if (stream->open(fileName, width, height, fps))
+        return stream;
+
+    stream->close();
+    free(stream);
+
+    return 0;
+}
+
+void release_OutputMediaStream_FFMPEG(struct OutputMediaStream_FFMPEG* stream)
+{
+    stream->close();
+    free(stream);
+}
+
+void write_OutputMediaStream_FFMPEG(struct OutputMediaStream_FFMPEG* stream, unsigned char* data, int size, int keyFrame)
+{
+    stream->write(data, size, keyFrame);
+}
+
+/*
+ * For CUDA decoder
+ */
+
+enum
+{
+    VideoCodec_MPEG1 = 0,
+    VideoCodec_MPEG2,
+    VideoCodec_MPEG4,
+    VideoCodec_VC1,
+    VideoCodec_H264,
+    VideoCodec_JPEG,
+    VideoCodec_H264_SVC,
+    VideoCodec_H264_MVC,
+
+    // Uncompressed YUV
+    VideoCodec_YUV420 = (('I'<<24)|('Y'<<16)|('U'<<8)|('V')),   // Y,U,V (4:2:0)
+    VideoCodec_YV12   = (('Y'<<24)|('V'<<16)|('1'<<8)|('2')),   // Y,V,U (4:2:0)
+    VideoCodec_NV12   = (('N'<<24)|('V'<<16)|('1'<<8)|('2')),   // Y,UV  (4:2:0)
+    VideoCodec_YUYV   = (('Y'<<24)|('U'<<16)|('Y'<<8)|('V')),   // YUYV/YUY2 (4:2:2)
+    VideoCodec_UYVY   = (('U'<<24)|('Y'<<16)|('V'<<8)|('Y')),   // UYVY (4:2:2)
+};
+
+enum
+{
+    VideoChromaFormat_Monochrome = 0,
+    VideoChromaFormat_YUV420,
+    VideoChromaFormat_YUV422,
+    VideoChromaFormat_YUV444,
+};
+
+struct InputMediaStream_FFMPEG
+{
+public:
+    bool open(const char* fileName, int* codec, int* chroma_format, int* width, int* height);
+    void close();
+
+    bool read(unsigned char** data, int* size, int* endOfFile);
+
+private:
+    InputMediaStream_FFMPEG(const InputMediaStream_FFMPEG&);
+    InputMediaStream_FFMPEG& operator =(const InputMediaStream_FFMPEG&);
+
+    AVFormatContext* ctx_;
+    int video_stream_id_;
+    AVPacket pkt_;
+};
+
+bool InputMediaStream_FFMPEG::open(const char* fileName, int* codec, int* chroma_format, int* width, int* height)
+{
+    int err;
+
+    ctx_ = 0;
+    video_stream_id_ = -1;
+    memset(&pkt_, 0, sizeof(AVPacket));
+
+    #if LIBAVFORMAT_BUILD >= CALC_FFMPEG_VERSION(53, 13, 0)
+        avformat_network_init();
+    #endif
+
+    #if LIBAVFORMAT_BUILD >= CALC_FFMPEG_VERSION(53, 6, 0)
+        err = avformat_open_input(&ctx_, fileName, 0, 0);
+    #else
+        err = av_open_input_file(&ctx_, fileName, 0, 0, 0);
+    #endif
+    if (err < 0)
+        return false;
+
+    #if LIBAVFORMAT_BUILD >= CALC_FFMPEG_VERSION(53, 3, 0)
+        err = avformat_find_stream_info(ctx_, 0);
+    #else
+        err = av_find_stream_info(ctx_);
+    #endif
+    if (err < 0)
+        return false;
+
+    for (unsigned int i = 0; i < ctx_->nb_streams; ++i)
+    {
+        #if LIBAVFORMAT_BUILD > 4628
+            AVCodecContext *enc = ctx_->streams[i]->codec;
+        #else
+            AVCodecContext *enc = &ctx_->streams[i]->codec;
+        #endif
+
+        if (enc->codec_type == AVMEDIA_TYPE_VIDEO)
+        {
+            video_stream_id_ = static_cast<int>(i);
+
+            switch (enc->codec_id)
+            {
+            case CV_CODEC(CODEC_ID_MPEG1VIDEO):
+                *codec = ::VideoCodec_MPEG1;
+                break;
+
+            case CV_CODEC(CODEC_ID_MPEG2VIDEO):
+                *codec = ::VideoCodec_MPEG2;
+                break;
+
+            case CV_CODEC(CODEC_ID_MPEG4):
+                *codec = ::VideoCodec_MPEG4;
+                break;
+
+            case CV_CODEC(CODEC_ID_VC1):
+                *codec = ::VideoCodec_VC1;
+                break;
+
+            case CV_CODEC(CODEC_ID_H264):
+                *codec = ::VideoCodec_H264;
+                break;
+
+            default:
+                return false;
+            };
+
+            switch (enc->pix_fmt)
+            {
+            case PIX_FMT_YUV420P:
+                *chroma_format = ::VideoChromaFormat_YUV420;
+                break;
+
+            case PIX_FMT_YUV422P:
+                *chroma_format = ::VideoChromaFormat_YUV422;
+                break;
+
+            case PIX_FMT_YUV444P:
+                *chroma_format = ::VideoChromaFormat_YUV444;
+                break;
+
+            default:
+                return false;
+            }
+
+            *width = enc->coded_width;
+            *height = enc->coded_height;
+
+            break;
+        }
+    }
+
+    if (video_stream_id_ < 0)
+        return false;
+
+    av_init_packet(&pkt_);
+
+    return true;
+}
+
+void InputMediaStream_FFMPEG::close()
+{
+    if (ctx_)
+    {
+        #if LIBAVFORMAT_BUILD >= CALC_FFMPEG_VERSION(53, 24, 2)
+            avformat_close_input(&ctx_);
+        #else
+            av_close_input_file(ctx_);
+        #endif
+    }
+
+    // free last packet if exist
+    if (pkt_.data)
+        av_free_packet(&pkt_);
+}
+
+bool InputMediaStream_FFMPEG::read(unsigned char** data, int* size, int* endOfFile)
+{
+    // free last packet if exist
+    if (pkt_.data)
+        av_free_packet(&pkt_);
+
+    // get the next frame
+    for (;;)
+    {
+        int ret = av_read_frame(ctx_, &pkt_);
+
+        if (ret == AVERROR(EAGAIN))
+            continue;
+
+        if (ret < 0)
+        {
+            if (ret == (int)AVERROR_EOF)
+                *endOfFile = true;
+            return false;
+        }
+
+        if (pkt_.stream_index != video_stream_id_)
+        {
+            av_free_packet(&pkt_);
+            continue;
+        }
+
+        break;
+    }
+
+    *data = pkt_.data;
+    *size = pkt_.size;
+    *endOfFile = false;
+
+    return true;
+}
+
+InputMediaStream_FFMPEG* create_InputMediaStream_FFMPEG(const char* fileName, int* codec, int* chroma_format, int* width, int* height)
+{
+    InputMediaStream_FFMPEG* stream = (InputMediaStream_FFMPEG*) malloc(sizeof(InputMediaStream_FFMPEG));
+
+    if (stream && stream->open(fileName, codec, chroma_format, width, height))
+        return stream;
+
+    stream->close();
+    free(stream);
+
+    return 0;
+}
+
+void release_InputMediaStream_FFMPEG(InputMediaStream_FFMPEG* stream)
+{
+    stream->close();
+    free(stream);
+}
+
+int read_InputMediaStream_FFMPEG(InputMediaStream_FFMPEG* stream, unsigned char** data, int* size, int* endOfFile)
+{
+    return stream->read(data, size, endOfFile);
+}
