@@ -346,7 +346,11 @@ static int numCameras = 0;
 static int indexList = 0;
 
 // IOCTL handling for V4L2
+#ifdef HAVE_IOCTL_ULONG
+static int xioctl( int fd, unsigned long request, void *arg)
+#else
 static int xioctl( int fd, int request, void *arg)
+#endif
 {
 
   int r;
@@ -956,6 +960,7 @@ static int _capture_V4L (CvCaptureCAM_V4L *capture, char *deviceName)
      if (capture->memoryMap == MAP_FAILED) {
         fprintf( stderr, "HIGHGUI ERROR: V4L: Mapping Memmory from video source error: %s\n", strerror(errno));
         icvCloseCAM_V4L(capture);
+        return -1;
      }
 
      /* Set up video_mmap structure pointing to this memory mapped area so each image may be
@@ -1007,10 +1012,6 @@ static CvCaptureCAM_V4L * icvCaptureFromCAM_V4L (int index)
       return NULL;
    }
 
-   /* set the default size */
-   capture->width  = DEFAULT_V4L_WIDTH;
-   capture->height = DEFAULT_V4L_HEIGHT;
-
 #ifdef USE_TEMP_BUFFER
    capture->buffers[MAX_V4L_BUFFERS].start = NULL;
 #endif
@@ -1034,6 +1035,9 @@ static CvCaptureCAM_V4L * icvCaptureFromCAM_V4L (int index)
       the standard set of cv calls promoting transparency.  "Vector Table" insertion. */
    capture->FirstCapture = 1;
 
+   /* set the default size */
+   capture->width  = DEFAULT_V4L_WIDTH;
+   capture->height = DEFAULT_V4L_HEIGHT;
 
    if (_capture_V4L2 (capture, deviceName) == -1) {
        icvCloseCAM_V4L(capture);
@@ -1110,7 +1114,7 @@ static void mainloop_v4l2(CvCaptureCAM_V4L* capture) {
             FD_SET (capture->deviceHandle, &fds);
 
             /* Timeout. */
-            tv.tv_sec = 2;
+            tv.tv_sec = 10;
             tv.tv_usec = 0;
 
             r = select (capture->deviceHandle+1, &fds, NULL, NULL, &tv);
@@ -1709,6 +1713,8 @@ static void icvCloseCAM_V4L( CvCaptureCAM_V4L* capture ){
      }
 #endif
 
+     free(capture->deviceName);
+     capture->deviceName = NULL;
      //v4l2_free_ranges(capture);
      //cvFree((void **)capture);
    }
