@@ -13,7 +13,7 @@ typedef perf::TestBaseWithParam<Img_BlockSize_ApertureSize_BorderType_t> Img_Blo
 
 PERF_TEST_P(Img_BlockSize_ApertureSize_BorderType, cornerEigenValsAndVecs,
             testing::Combine(
-                testing::Values( "stitching/a1.jpg", "cv/shared/pic5.png"),
+                testing::Values( "stitching/a1.png", "cv/shared/pic5.png"),
                 testing::Values( 3, 5 ),
                 testing::Values( 3, 5 ),
                 testing::ValuesIn(BorderType::all())
@@ -28,7 +28,7 @@ PERF_TEST_P(Img_BlockSize_ApertureSize_BorderType, cornerEigenValsAndVecs,
     Mat src = imread(filename, IMREAD_GRAYSCALE);
     if (src.empty())
         FAIL() << "Unable to load source image" << filename;
-    
+
     Mat dst;
 
     TEST_CYCLE() cornerEigenValsAndVecs(src, dst, blockSize, apertureSize, borderType);
