@@ -1,16 +1,6 @@
-#include <jni.h>
-
-#include "converters.h"
-
-#include <android/log.h>
 #define LOG_TAG "org.opencv.core.Mat"
-#define LOGE(...) ((void)__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, __VA_ARGS__))
-#ifdef DEBUG
-#define LOGD(...) ((void)__android_log_print(ANDROID_LOG_DEBUG, LOG_TAG, __VA_ARGS__))
-#else //!DEBUG
-#define LOGD(...)
-#endif //DEBUG
 
+#include "common.h"
 #include "opencv2/core/core.hpp"
 
 using namespace cv;
@@ -2163,7 +2153,7 @@ template<typename T> static int mat_put(cv::Mat* m, int row, int col, int count,
     if(! buff) return 0;
 
     count *= sizeof(T);
-    int rest = ((m->rows - row) * m->cols - col) * m->elemSize();
+    int rest = ((m->rows - row) * m->cols - col) * (int)m->elemSize();
     if(count>rest) count = rest;
     int res = count;
 
@@ -2172,14 +2162,14 @@ template<typename T> static int mat_put(cv::Mat* m, int row, int col, int count,
         memcpy(m->ptr(row, col), buff, count);
     } else {
         // row by row
-        int num = (m->cols - col) * m->elemSize(); // 1st partial row
+        int num = (m->cols - col) * (int)m->elemSize(); // 1st partial row
         if(count<num) num = count;
         uchar* data = m->ptr(row++, col);
         while(count>0){
             memcpy(data, buff, num);
             count -= num;
             buff += num;
-            num = m->cols * m->elemSize();
+            num = m->cols * (int)m->elemSize();
             if(count<num) num = count;
             data = m->ptr(row++, 0);
         }
@@ -2323,7 +2313,7 @@ template<typename T> int mat_get(cv::Mat* m, int row, int col, int count, char*
     if(! buff) return 0;
 
     int bytesToCopy = count * sizeof(T);
-    int bytesRestInMat = ((m->rows - row) * m->cols - col) * m->elemSize();
+    int bytesRestInMat = ((m->rows - row) * m->cols - col) * (int)m->elemSize();
     if(bytesToCopy > bytesRestInMat) bytesToCopy = bytesRestInMat;
     int res = bytesToCopy;
 
@@ -2332,7 +2322,7 @@ template<typename T> int mat_get(cv::Mat* m, int row, int col, int count, char*
         memcpy(buff, m->ptr(row, col), bytesToCopy);
     } else {
         // row by row
-        int bytesInRow = (m->cols - col) * m->elemSize(); // 1st partial row
+        int bytesInRow = (m->cols - col) * (int)m->elemSize(); // 1st partial row
         while(bytesToCopy > 0)
         {
             int len = std::min(bytesToCopy, bytesInRow);
@@ -2341,7 +2331,7 @@ template<typename T> int mat_get(cv::Mat* m, int row, int col, int count, char*
             buff += len;
             row++;
             col = 0;
-            bytesInRow = m->cols * m->elemSize();
+            bytesInRow = m->cols * (int)m->elemSize();
         }
     }
     return res;
@@ -2518,7 +2508,7 @@ JNIEXPORT jdoubleArray JNICALL Java_org_opencv_core_Mat_nGet
 
         jdoubleArray res = env->NewDoubleArray(me->channels());
         if(res){
-            jdouble buff[me->channels()];
+            jdouble buff[CV_CN_MAX];//me->channels()
             int i;
             switch(me->depth()){
                 case CV_8U:  for(i=0; i<me->channels(); i++) buff[i] = *((unsigned char*) me->ptr(row, col) + i); break;
