@@ -1,4 +1,4 @@
-#include <jni.h>
+#include "common.h"
 
 #include "opencv2/opencv_modules.hpp"
 
@@ -18,6 +18,10 @@
 #  include "opencv2/ml/ml.hpp"
 #endif
 
+#ifdef HAVE_OPENCV_CONTRIB
+#  include "opencv2/contrib/contrib.hpp"
+#endif
+
 extern "C" {
 
 JNIEXPORT jint JNICALL
@@ -40,6 +44,9 @@ JNI_OnLoad(JavaVM* vm, void* )
 #ifdef HAVE_OPENCV_ML
     init &= cv::initModule_ml();
 #endif
+    #ifdef HAVE_OPENCV_CONTRIB
+    init &= cv::initModule_contrib();
+#endif
 
     if(!init)
         return -1;
