@@ -235,7 +235,7 @@ void CvCalibFilter::SetCameraCount( int count )
             cvReleaseMat( &rectMap[i][1] );
         }
 
-        memset( latestCounts, 0, sizeof(latestPoints) );
+        memset( latestCounts, 0, sizeof(latestCounts) );
         maxPoints = 0;
         cameraCount = count;
     }
