@@ -44,6 +44,73 @@
 
 #include "cascadedetect.hpp"
 
+#include <string>
+
+#if defined (LOG_CASCADE_STATISTIC)
+struct Logger
+{
+    enum { STADIES_NUM = 20 };
+
+    int gid;
+    cv::Mat mask;
+    cv::Size sz0;
+    int step;
+
+
+    Logger() : gid (0), step(2) {}
+    void setImage(const cv::Mat& image)
+    {
+     if (gid == 0)
+         sz0 = image.size();
+
+      mask.create(image.rows, image.cols * (STADIES_NUM + 1) + STADIES_NUM, CV_8UC1);
+      mask = cv::Scalar(0);
+      cv::Mat roi = mask(cv::Rect(cv::Point(0,0), image.size()));
+      image.copyTo(roi);
+
+      printf("%d) Size = (%d, %d)\n", gid, image.cols, image.rows);
+
+      for(int i = 0; i < STADIES_NUM; ++i)
+      {
+          int x = image.cols + i * (image.cols + 1);
+          cv::line(mask, cv::Point(x, 0), cv::Point(x, mask.rows-1), cv::Scalar(255));
+      }
+
+      if (sz0.width/image.cols > 2 && sz0.height/image.rows > 2)
+          step = 1;
+    }
+
+    void setPoint(const cv::Point& p, int passed_stadies)
+    {
+        int cols = mask.cols / (STADIES_NUM + 1);
+
+        passed_stadies = -passed_stadies;
+        passed_stadies = (passed_stadies == -1) ? STADIES_NUM : passed_stadies;
+
+        unsigned char* ptr = mask.ptr<unsigned char>(p.y) + cols + 1 + p.x;
+        for(int i = 0; i < passed_stadies; ++i, ptr += cols + 1)
+        {
+            *ptr = 255;
+
+            if (step == 2)
+            {
+                ptr[1] = 255;
+                ptr[mask.step] = 255;
+                ptr[mask.step + 1] = 255;
+            }
+        }
+    };
+
+    void write()
+    {
+        char buf[4096];
+        sprintf(buf, "%04d.png", gid++);
+        cv::imwrite(buf, mask);
+    }
+
+} logger;
+#endif
+
 namespace cv
 {
 
@@ -170,7 +237,7 @@ class MeanshiftGrouping
 {
 public:
     MeanshiftGrouping(const Point3d& densKer, const vector<Point3d>& posV,
-        const vector<double>& wV, double, int maxIter = 20)
+        const vector<double>& wV, double eps, int maxIter = 20)
     {
         densityKernel = densKer;
         weightsV = wV;
@@ -179,6 +246,7 @@ public:
         meanshiftV.resize(positionsCount);
         distanceV.resize(positionsCount);
         iterMax = maxIter;
+        modeEps = eps;
 
         for (unsigned i = 0; i<positionsV.size(); i++)
         {
@@ -876,10 +944,11 @@ void CascadeClassifier::setFaceDetectionMaskGenerator()
 #endif
 }
 
-struct CascadeClassifierInvoker
+class CascadeClassifierInvoker : public ParallelLoopBody
 {
+public:
     CascadeClassifierInvoker( CascadeClassifier& _cc, Size _sz1, int _stripSize, int _yStep, double _factor,
-        ConcurrentRectVector& _vec, vector<int>& _levels, vector<double>& _weights, bool outputLevels, const Mat& _mask)
+        vector<Rect>& _vec, vector<int>& _levels, vector<double>& _weights, bool outputLevels, const Mat& _mask, Mutex* _mtx)
     {
         classifier = &_cc;
         processingRectSize = _sz1;
@@ -887,19 +956,20 @@ struct CascadeClassifierInvoker
         yStep = _yStep;
         scalingFactor = _factor;
         rectangles = &_vec;
-        rejectLevels  = outputLevels ? &_levels : 0;
-        levelWeights  = outputLevels ? &_weights : 0;
-        mask=_mask;
+        rejectLevels = outputLevels ? &_levels : 0;
+        levelWeights = outputLevels ? &_weights : 0;
+        mask = _mask;
+        mtx = _mtx;
     }
 
-    void operator()(const BlockedRange& range) const
+    void operator()(const Range& range) const
     {
         Ptr<FeatureEvaluator> evaluator = classifier->featureEvaluator->clone();
 
         Size winSize(cvRound(classifier->data.origWinSize.width * scalingFactor), cvRound(classifier->data.origWinSize.height * scalingFactor));
 
-        int y1 = range.begin() * stripSize;
-        int y2 = min(range.end() * stripSize, processingRectSize.height);
+        int y1 = range.start * stripSize;
+        int y2 = min(range.end * stripSize, processingRectSize.height);
         for( int y = y1; y < y2; y += yStep )
         {
             for( int x = 0; x < processingRectSize.width; x += yStep )
@@ -910,20 +980,31 @@ struct CascadeClassifierInvoker
 
                 double gypWeight;
                 int result = classifier->runAt(evaluator, Point(x, y), gypWeight);
+
+#if defined (LOG_CASCADE_STATISTIC)
+
+                logger.setPoint(Point(x, y), result);
+#endif
                 if( rejectLevels )
                 {
                     if( result == 1 )
                         result =  -(int)classifier->data.stages.size();
                     if( classifier->data.stages.size() + result < 4 )
                     {
+                        mtx->lock();
                         rectangles->push_back(Rect(cvRound(x*scalingFactor), cvRound(y*scalingFactor), winSize.width, winSize.height));
                         rejectLevels->push_back(-result);
                         levelWeights->push_back(gypWeight);
+                        mtx->unlock();
                     }
                 }
                 else if( result > 0 )
+                {
+                    mtx->lock();
                     rectangles->push_back(Rect(cvRound(x*scalingFactor), cvRound(y*scalingFactor),
                                                winSize.width, winSize.height));
+                    mtx->unlock();
+                }
                 if( result == 0 )
                     x += yStep;
             }
@@ -931,17 +1012,19 @@ struct CascadeClassifierInvoker
     }
 
     CascadeClassifier* classifier;
-    ConcurrentRectVector* rectangles;
+    vector<Rect>* rectangles;
     Size processingRectSize;
     int stripSize, yStep;
     double scalingFactor;
     vector<int> *rejectLevels;
     vector<double> *levelWeights;
     Mat mask;
+    Mutex* mtx;
 };
 
 struct getRect { Rect operator ()(const CvAvgComp& e) const { return e.rect; } };
 
+
 bool CascadeClassifier::detectSingleScale( const Mat& image, int stripCount, Size processingRectSize,
                                            int stripSize, int yStep, double factor, vector<Rect>& candidates,
                                            vector<int>& levels, vector<double>& weights, bool outputRejectLevels )
@@ -949,27 +1032,36 @@ bool CascadeClassifier::detectSingleScale( const Mat& image, int stripCount, Siz
     if( !featureEvaluator->setImage( image, data.origWinSize ) )
         return false;
 
+#if defined (LOG_CASCADE_STATISTIC)
+    logger.setImage(image);
+#endif
+
     Mat currentMask;
     if (!maskGenerator.empty()) {
         currentMask=maskGenerator->generateMask(image);
     }
 
-    ConcurrentRectVector concurrentCandidates;
+    vector<Rect> candidatesVector;
     vector<int> rejectLevels;
     vector<double> levelWeights;
+    Mutex mtx;
     if( outputRejectLevels )
     {
-        parallel_for(BlockedRange(0, stripCount), CascadeClassifierInvoker( *this, processingRectSize, stripSize, yStep, factor,
-            concurrentCandidates, rejectLevels, levelWeights, true, currentMask));
+        parallel_for_(Range(0, stripCount), CascadeClassifierInvoker( *this, processingRectSize, stripSize, yStep, factor,
+            candidatesVector, rejectLevels, levelWeights, true, currentMask, &mtx));
         levels.insert( levels.end(), rejectLevels.begin(), rejectLevels.end() );
         weights.insert( weights.end(), levelWeights.begin(), levelWeights.end() );
     }
     else
     {
-         parallel_for(BlockedRange(0, stripCount), CascadeClassifierInvoker( *this, processingRectSize, stripSize, yStep, factor,
-            concurrentCandidates, rejectLevels, levelWeights, false, currentMask));
+         parallel_for_(Range(0, stripCount), CascadeClassifierInvoker( *this, processingRectSize, stripSize, yStep, factor,
+            candidatesVector, rejectLevels, levelWeights, false, currentMask, &mtx));
     }
-    candidates.insert( candidates.end(), concurrentCandidates.begin(), concurrentCandidates.end() );
+    candidates.insert( candidates.end(), candidatesVector.begin(), candidatesVector.end() );
+
+#if defined (LOG_CASCADE_STATISTIC)
+    logger.write();
+#endif
 
     return true;
 }
