@@ -1,3 +1,5 @@
+#/usr/bin/env python
+
 import hdr_parser, sys, re, os, cStringIO
 from string import Template
 
@@ -6,6 +8,11 @@ gen_template_check_self = Template("""    if(!PyObject_TypeCheck(self, &pyopencv
     $cname* _self_ = ${amp}((pyopencv_${name}_t*)self)->v;
 """)
 
+gen_template_check_self_algo = Template("""    if(!PyObject_TypeCheck(self, &pyopencv_${name}_Type))
+        return failmsgp("Incorrect type of self (must be '${name}' or its derivative)");
+    $cname* _self_ = dynamic_cast<$cname*>(${amp}((pyopencv_${name}_t*)self)->v.obj);
+""")
+
 gen_template_call_constructor = Template("""self = PyObject_NEW(pyopencv_${name}_t, &pyopencv_${name}_Type);
         new (&(self->v)) Ptr<$cname>(); // init Ptr with placement new
         if(self) ERRWRAP2(self->v = new $cname""")
@@ -70,7 +77,7 @@ gen_template_type_decl = Template("""
 struct pyopencv_${name}_t
 {
     PyObject_HEAD
-    Ptr<${cname}> v;
+    Ptr<${cname1}> v;
 };
 
 static PyTypeObject pyopencv_${name}_Type =
@@ -83,14 +90,14 @@ static PyTypeObject pyopencv_${name}_Type =
 
 static void pyopencv_${name}_dealloc(PyObject* self)
 {
-    ((pyopencv_${name}_t*)self)->v = NULL;
+    ((pyopencv_${name}_t*)self)->v.release();
     PyObject_Del(self);
 }
 
 static PyObject* pyopencv_from(const Ptr<${cname}>& r)
 {
     pyopencv_${name}_t *m = PyObject_NEW(pyopencv_${name}_t, &pyopencv_${name}_Type);
-    new (&(m->v)) Ptr<$cname>(); // init Ptr with placement new
+    new (&(m->v)) Ptr<$cname1>(); // init Ptr with placement new
     m->v = r;
     return (PyObject*)m;
 }
@@ -164,6 +171,13 @@ static PyObject* pyopencv_${name}_get_${member}(pyopencv_${name}_t* p, void *clo
 }
 """)
 
+gen_template_get_prop_algo = Template("""
+static PyObject* pyopencv_${name}_get_${member}(pyopencv_${name}_t* p, void *closure)
+{
+    return pyopencv_from(dynamic_cast<$cname*>(p->v.obj)${access}${member});
+}
+""")
+
 gen_template_set_prop = Template("""
 static int pyopencv_${name}_set_${member}(pyopencv_${name}_t* p, PyObject *value, void *closure)
 {
@@ -176,6 +190,19 @@ static int pyopencv_${name}_set_${member}(pyopencv_${name}_t* p, PyObject *value
 }
 """)
 
+gen_template_set_prop_algo = Template("""
+static int pyopencv_${name}_set_${member}(pyopencv_${name}_t* p, PyObject *value, void *closure)
+{
+    if (value == NULL)
+    {
+        PyErr_SetString(PyExc_TypeError, "Cannot delete the ${member} attribute");
+        return -1;
+    }
+    return pyopencv_to(value, dynamic_cast<$cname*>(p->v.obj)${access}${member}) ? 0 : -1;
+}
+""")
+
+
 gen_template_prop_init = Template("""
     {(char*)"${member}", (getter)pyopencv_${name}_get_${member}, NULL, (char*)"${member}", NULL},""")
 
@@ -207,6 +234,7 @@ class ClassInfo(object):
         self.name = self.wname = normalize_class_name(name)
         self.ismap = False
         self.issimple = False
+        self.isalgorithm = False
         self.methods = {}
         self.props = []
         self.consts = {}
@@ -215,13 +243,15 @@ class ClassInfo(object):
         if decl:
             self.bases = decl[1].split()[1:]
             if len(self.bases) > 1:
-                print "Warning: class %s has more than 1 base class (not supported by Python C extensions)" % (self.name,)
-                print "Bases: ", " ".join(self.bases)
-                print "Only the first base class will be used"
+                print "Note: Class %s has more than 1 base class (not supported by Python C extensions)" % (self.name,)
+                print "      Bases: ", " ".join(self.bases)
+                print "      Only the first base class will be used"
                 self.bases = [self.bases[0].strip(",")]
                 #return sys.exit(-1)
             if self.bases and self.bases[0].startswith("cv::"):
                 self.bases[0] = self.bases[0][4:]
+            if self.bases and self.bases[0] == "Algorithm":
+                self.isalgorithm = True
             for m in decl[2]:
                 if m.startswith("="):
                     self.wname = m[1:]
@@ -259,11 +289,17 @@ class ClassInfo(object):
             access_op = "."
 
         for pname, p in sorted_props:
-            getset_code.write(gen_template_get_prop.substitute(name=self.name, member=pname, membertype=p.tp, access=access_op))
+            if self.isalgorithm:
+                getset_code.write(gen_template_get_prop_algo.substitute(name=self.name, cname=self.cname, member=pname, membertype=p.tp, access=access_op))
+            else:
+                getset_code.write(gen_template_get_prop.substitute(name=self.name, member=pname, membertype=p.tp, access=access_op))
             if p.readonly:
                 getset_inits.write(gen_template_prop_init.substitute(name=self.name, member=pname))
             else:
-                getset_code.write(gen_template_set_prop.substitute(name=self.name, member=pname, membertype=p.tp, access=access_op))
+                if self.isalgorithm:
+                    getset_code.write(gen_template_set_prop_algo.substitute(name=self.name, cname=self.cname, member=pname, membertype=p.tp, access=access_op))
+                else:
+                    getset_code.write(gen_template_set_prop.substitute(name=self.name, member=pname, membertype=p.tp, access=access_op))
                 getset_inits.write(gen_template_rw_prop_init.substitute(name=self.name, member=pname))
 
         methods_code = cStringIO.StringIO()
@@ -329,6 +365,9 @@ class ArgInfo(object):
     def isbig(self):
         return self.tp == "Mat" or self.tp == "vector_Mat"# or self.tp.startswith("vector")
 
+    def crepr(self):
+        return "ArgInfo(\"%s\", %d)" % (self.name, self.outputarg)
+
 
 class FuncVariant(object):
     def __init__(self, classname, name, decl, isconstructor):
@@ -507,7 +546,10 @@ class FuncInfo(object):
                 amp = ""
                 if selfinfo.issimple:
                     amp = "&"
-                code += gen_template_check_self.substitute(name=selfinfo.name, cname=selfinfo.cname, amp=amp)
+                if selfinfo.isalgorithm:
+                    code += gen_template_check_self_algo.substitute(name=selfinfo.name, cname=selfinfo.cname, amp=amp)
+                else:
+                    code += gen_template_check_self.substitute(name=selfinfo.name, cname=selfinfo.cname, amp=amp)
                 fullname = selfinfo.wname + "." + fullname
 
         all_code_variants = []
@@ -561,7 +603,7 @@ class FuncInfo(object):
                     if amapping[1] == "O":
                         code_decl += "    PyObject* pyobj_%s = NULL;\n" % (a.name,)
                         parse_name = "pyobj_" + a.name
-                        code_cvt_list.append("pyopencv_to(pyobj_%s, %s)" % (a.name, a.name))
+                        code_cvt_list.append("pyopencv_to(pyobj_%s, %s, %s)" % (a.name, a.name, a.crepr()))
 
                 all_cargs.append([amapping, parse_name])
 
@@ -672,6 +714,8 @@ class PythonWrapperGenerator(object):
                 % (classinfo.name, classinfo.cname)
             sys.exit(-1)
         self.classes[classinfo.name] = classinfo
+        if classinfo.bases and not classinfo.isalgorithm:
+            classinfo.isalgorithm = self.classes[classinfo.bases[0]].isalgorithm
 
     def add_const(self, name, decl):
         constinfo = ConstInfo(name, decl[1])
@@ -767,7 +811,8 @@ class PythonWrapperGenerator(object):
                     templ = gen_template_simple_type_decl
                 else:
                     templ = gen_template_type_decl
-                self.code_types.write(templ.substitute(name=name, wname=classinfo.wname, cname=classinfo.cname))
+                self.code_types.write(templ.substitute(name=name, wname=classinfo.wname, cname=classinfo.cname,
+                                      cname1=("cv::Algorithm" if classinfo.isalgorithm else classinfo.cname)))
 
         # register classes in the same order as they have been declared.
         # this way, base classes will be registered in Python before their derivatives.
@@ -810,6 +855,3 @@ if __name__ == "__main__":
         srcfiles = sys.argv[2:]
     generator = PythonWrapperGenerator()
     generator.gen(srcfiles, dstdir)
-
-
-
