@@ -1,3 +1,5 @@
+#/usr/bin/env python
+
 import os, sys, re, string
 
 # the list only for debugging. The real list, used in the real OpenCV build, is specified in CMakeLists.txt
@@ -11,6 +13,7 @@ opencv_hdr_list = [
 "../../video/include/opencv2/video/tracking.hpp",
 "../../video/include/opencv2/video/background_segm.hpp",
 "../../objdetect/include/opencv2/objdetect/objdetect.hpp",
+"../../contrib/include/opencv2/contrib/contrib.hpp",
 "../../highgui/include/opencv2/highgui/highgui.hpp"
 ]
 
@@ -154,6 +157,8 @@ class CppHeaderParser(object):
             elif angle_stack:
                 arg_type += w
                 angle_stack[-1] += 1
+            elif arg_type == "struct":
+                arg_type += " " + w
             elif arg_type and arg_type != "~":
                 arg_name = " ".join(word_list[wi:])
                 break
@@ -332,10 +337,10 @@ class CppHeaderParser(object):
                     atype = arg[:pos+1].strip()
                     if aname.endswith("&") or aname.endswith("*") or (aname in ["int", "string", "Mat"]):
                         atype = (atype + " " + aname).strip()
-                        aname = "param"
+                        aname = ""
                 else:
                     atype = arg
-                    aname = "param"
+                    aname = ""
                 if aname.endswith("]"):
                     bidx = aname.find('[')
                     atype += aname[bidx:]
@@ -428,7 +433,7 @@ class CppHeaderParser(object):
                 decl_start = decl_start[0:-2].rstrip() + " ()"
 
         # constructor/destructor case
-        if bool(re.match(r'(\w+::)*(?P<x>\w+)::~?(?P=x)', decl_start)):
+        if bool(re.match(r'^(\w+::)*(?P<x>\w+)::~?(?P=x)$', decl_start)):
             decl_start = "void " + decl_start
 
         rettype, funcname, modlist, argno = self.parse_arg(decl_start, -1)
@@ -440,10 +445,14 @@ class CppHeaderParser(object):
             else:
                 if bool(re.match('\w+\s+\(\*\w+\)\s*\(.*\)', decl_str)):
                     return [] # function typedef
+                elif bool(re.match('\w+\s+\(\w+::\*\w+\)\s*\(.*\)', decl_str)):
+                    return [] # class method typedef
                 elif bool(re.match('[A-Z_]+', decl_start)):
                     return [] # it seems to be a macro instantiation
                 elif "__declspec" == decl_start:
                     return []
+                elif bool(re.match(r'\w+\s+\(\*\w+\)\[\d+\]', decl_str)):
+                    return [] # exotic - dynamic 2d array
                 else:
                     #print rettype, funcname, modlist, argno
                     print "Error at %s:%d the function/method name is missing: '%s'" % (self.hname, self.lineno, decl_start)
@@ -566,6 +575,8 @@ class CppHeaderParser(object):
         """
         if not self.block_stack:
             return name
+        if name.startswith("cv."):
+            return name
         n = ""
         for b in self.block_stack:
             block_type, block_name = b[self.BLOCK_TYPE], b[self.BLOCK_NAME]
@@ -798,6 +809,10 @@ class CppHeaderParser(object):
                 stmt = " ".join(stmt.split()) # normalize the statement
                 stack_top = self.block_stack[-1]
 
+                if stmt.startswith("@"):
+                    # Objective C ?
+                    break
+
                 decl = None
                 if stack_top[self.PROCESS_FLAG]:
                     # even if stack_top[PUBLIC_SECTION] is False, we still try to process the statement,
@@ -849,5 +864,7 @@ if __name__ == '__main__':
     decls = []
     for hname in opencv_hdr_list:
         decls += parser.parse(hname)
+    #for hname in sys.argv[1:]:
+        #decls += parser.parse(hname, wmode=False)
     parser.print_decls(decls)
     print len(decls)
