@@ -1,3 +1,5 @@
+#/usr/bin/env python
+
 import unittest
 import random
 import time
@@ -14,6 +16,8 @@ import functools
 
 import cv2.cv as cv
 
+from test2 import *
+
 class OpenCVTests(unittest.TestCase):
 
     depths = [ cv.IPL_DEPTH_8U, cv.IPL_DEPTH_8S, cv.IPL_DEPTH_16U, cv.IPL_DEPTH_16S, cv.IPL_DEPTH_32S, cv.IPL_DEPTH_32F, cv.IPL_DEPTH_64F ]
@@ -69,7 +73,7 @@ class OpenCVTests(unittest.TestCase):
 
     def get_sample(self, filename, iscolor = cv.CV_LOAD_IMAGE_COLOR):
         if not filename in self.image_cache:
-            filedata = urllib.urlopen("http://code.opencv.org/svn/opencv/trunk/opencv/" + filename).read()
+            filedata = urllib.urlopen("https://raw.github.com/Itseez/opencv/master/" + filename).read()
             imagefiledata = cv.CreateMatHeader(1, len(filedata), cv.CV_8UC1)
             cv.SetData(imagefiledata, filedata, len(filedata))
             self.image_cache[filename] = cv.DecodeImageM(imagefiledata, iscolor)
@@ -390,10 +394,7 @@ class FunctionTests(OpenCVTests):
     def test_DrawChessboardCorners(self):
         im = cv.CreateImage((512,512), cv.IPL_DEPTH_8U, 3)
         cv.SetZero(im)
-        cv.DrawChessboardCorners(im, (5, 5), [ (100,100) for i in range(5 * 5) ], 1)
-        self.assert_(cv.Sum(im)[0] > 0)
-
-        self.assertRaises(TypeError, lambda: cv.DrawChessboardCorners(im, (4, 5), [ (100,100) for i in range(5 * 5) ], 1))
+        cv.DrawChessboardCorners(im, (5, 5), [ ((i/5)*100+50,(i%5)*100+50) for i in range(5 * 5) ], 1)
 
     def test_ExtractSURF(self):
         img = self.get_sample("samples/c/lena.jpg", 0)
@@ -845,7 +846,7 @@ class FunctionTests(OpenCVTests):
                     self.assertEqual(cv.Sum(M)[0], r * c)
 
     def test_Threshold(self):
-	#""" directed test for bug 2790622 """
+    #""" directed test for bug 2790622 """
         src = self.get_sample("samples/c/lena.jpg", 0)
         results = set()
         for i in range(10):
@@ -956,7 +957,7 @@ class AreaTests(OpenCVTests):
                 for j in range(10):
                     self.assert_(all([c == chr(1) for c in cs[j].tostring()]))
 
-            # 
+            #
             m = numpy.identity(4, dtype = numpy.float32)
             m = cv.fromarray(m[:3, :3])
             rvec = cv.CreateMat(3, 1, cv.CV_32FC1)
@@ -964,7 +965,7 @@ class AreaTests(OpenCVTests):
             rvec[1,0] = 1
             rvec[2,0] = 1
             cv.Rodrigues2(rvec, m)
-	    #print m
+        #print m
 
         else:
             print "SKIPPING test_numpy - numpy support not built"
@@ -1040,11 +1041,11 @@ class AreaTests(OpenCVTests):
         cvm = cv.CreateMat(20, 10, cv.CV_32FC1)
 
     def test_depths(self):
-	#""" Make sure that the depth enums are unique """
+    #""" Make sure that the depth enums are unique """
         self.assert_(len(self.depths) == len(set(self.depths)))
 
     def test_leak(self):
-	#""" If CreateImage is not releasing image storage, then the loop below should use ~4GB of memory. """
+    #""" If CreateImage is not releasing image storage, then the loop below should use ~4GB of memory. """
         for i in range(64000):
             a = cv.CreateImage((1024,1024), cv.IPL_DEPTH_8U, 1)
         for i in range(64000):
@@ -1831,7 +1832,7 @@ class AreaTests(OpenCVTests):
         #self.snap(img)
 
     def test_create(self):
-	#""" CvCreateImage, CvCreateMat and the header-only form """
+    #""" CvCreateImage, CvCreateMat and the header-only form """
         for (w,h) in [ (320,400), (640,480), (1024, 768) ]:
             data = "z" * (w * h)
 
@@ -1910,7 +1911,7 @@ class AreaTests(OpenCVTests):
                 temp = cv.CloneImage(im)
                 for op in ["OPEN", "CLOSE", "GRADIENT", "TOPHAT", "BLACKHAT"]:
                         cv.MorphologyEx(im, dst, temp, e, eval("cv.CV_MOP_%s" % op), iter)
-        
+
     def test_getmat_nd(self):
         # 1D CvMatND should yield (N,1) CvMat
         matnd = cv.CreateMatND([13], cv.CV_8UC1)
@@ -2147,7 +2148,7 @@ class AreaTests(OpenCVTests):
         print mat
 
     def test_rand_PutText(self):
-	#""" Test for bug 2829336 """
+    #""" Test for bug 2829336 """
         mat = cv.CreateMat( 64, 64, cv.CV_8UC1)
         font = cv.InitFont(cv.CV_FONT_HERSHEY_SIMPLEX, 1, 1)
         cv.PutText(mat, chr(127), (20, 20), font, 255)
@@ -2201,10 +2202,6 @@ class DocumentFragmentTests(OpenCVTests):
         self.assertEqual(self.hashimg(h1), self.hashimg(h3))
         self.assertNotEqual(self.hashimg(h1), self.hashimg(h2))
 
-class NewTests(OpenCVTests):
-
-    pass
-
 if __name__ == '__main__':
     print "testing", cv.__version__
     random.seed(0)
