@@ -14,7 +14,7 @@ using namespace cv;
 /**
  * @function main
  */
-int main ( int argc, char** argv )
+int main ( int, char** argv )
 {
   /// Declare variables
   Mat src, dst;
@@ -22,9 +22,9 @@ int main ( int argc, char** argv )
   Mat kernel;
   Point anchor;
   double delta;
-  int ddepth;  
+  int ddepth;
   int kernel_size;
-  char* window_name = "filter2D Demo";
+  const char* window_name = "filter2D Demo";
 
   int c;
 
@@ -36,7 +36,7 @@ int main ( int argc, char** argv )
 
   /// Create window
   namedWindow( window_name, CV_WINDOW_AUTOSIZE );
-  
+
   /// Initialize arguments for the filter
   anchor = Point( -1, -1 );
   delta = 0;
@@ -44,7 +44,7 @@ int main ( int argc, char** argv )
 
   /// Loop - Will filter the image with different kernel sizes each 0.5 seconds
   int ind = 0;
-  while( true )
+  for(;;)
        {
          c = waitKey(500);
          /// Press 'ESC' to exit the program
@@ -60,6 +60,6 @@ int main ( int argc, char** argv )
          imshow( window_name, dst );
          ind++;
        }
-  
+
   return 0;
 }
